@@ -62,16 +62,16 @@ const utf8	WLButton::PushedBottomRightImageName[]		= "ButtonPushedBottomRight";
 const utf8	WLButton::MouseCursorImageName[]			= "MouseArrow";
 
 // colours
-const colour	WLButton::NormalPrimaryColour		= 0xAFAFAF;
-const colour	WLButton::NormalSecondaryColour		= 0xFFFFFF;
-const colour	WLButton::HoverPrimaryColour		= 0xCFD9CF;
-const colour	WLButton::HoverSecondaryColour		= 0xF2FFF2;
-const colour	WLButton::PushedPrimaryColour		= 0xAFAFAF;
-const colour	WLButton::PushedSecondaryColour		= 0xFFFFFF;
-const colour	WLButton::DisabledPrimaryColour		= 0x999999;
-const colour	WLButton::DisabledSecondaryColour	= 0x999999;
-const colour	WLButton::EnabledTextLabelColour	= 0x000000;
-const colour	WLButton::DisabledTextLabelColour	= 0x888888;
+const colour	WLButton::NormalPrimaryColour		= 0xFFAFAFAF;
+const colour	WLButton::NormalSecondaryColour		= 0xFFFFFFFF;
+const colour	WLButton::HoverPrimaryColour		= 0xFFCFD9CF;
+const colour	WLButton::HoverSecondaryColour		= 0xFFF2FFF2;
+const colour	WLButton::PushedPrimaryColour		= 0xFFAFAFAF;
+const colour	WLButton::PushedSecondaryColour		= 0xFFFFFFFF;
+const colour	WLButton::DisabledPrimaryColour		= 0xFF999999;
+const colour	WLButton::DisabledSecondaryColour	= 0xFF999999;
+const colour	WLButton::EnabledTextLabelColour	= 0xFF000000;
+const colour	WLButton::DisabledTextLabelColour	= 0xFF888888;
 
 	
 /*************************************************************************
@@ -80,14 +80,6 @@ const colour	WLButton::DisabledTextLabelColour	= 0x888888;
 WLButton::WLButton(const String& type, const String& name) :
 	PushButton(type, name)
 {
-	// default options
-	d_autoscaleImages		= true;
-	d_useStandardImagery	= true;
-	d_useNormalImage		= false;
-	d_useHoverImage			= false;
-	d_usePushedImage		= false;
-	d_useDisabledImage		= false;
-
 	storeFrameSizes();
 
 	// setup frames & image pointers
@@ -147,105 +139,6 @@ WLButton::~WLButton(void)
 }
 
 
-
-/*************************************************************************
-	set whether or not to render the standard imagery for the button	
-*************************************************************************/
-void WLButton::setStandardImageryEnabled(bool setting)
-{
-	if (d_useStandardImagery != setting)
-	{
-		d_useStandardImagery = setting;
-		requestRedraw();
-	}
-
-}
-
-
-/*************************************************************************
-	set the image to render for the button (normal state) - use NULL to
-	disable drawing of image	
-*************************************************************************/
-void WLButton::setNormalImage(const RenderableImage* image)
-{
-	if (image == NULL)
-	{
-		d_useNormalImage = false;
-	}
-	else
-	{
-		d_useNormalImage = true;
-		d_normalImage = *image;
-		d_normalImage.setRect(Rect(0, 0, d_abs_area.getWidth(), d_abs_area.getHeight()));
-	}
-
-	requestRedraw();
-}
-
-
-/*************************************************************************
-	set the image to render for the button (hover state)  - use NULL to
-	disable drawing of image
-*************************************************************************/
-void WLButton::setHoverImage(const RenderableImage* image)
-{
-	if (image == NULL)
-	{
-		d_useHoverImage = false;
-	}
-	else
-	{
-		d_useHoverImage = true;
-		d_hoverImage = *image;
-		d_hoverImage.setRect(Rect(0, 0, d_abs_area.getWidth(), d_abs_area.getHeight()));
-	}
-
-	requestRedraw();
-}
-
-
-/*************************************************************************
-	set the image to render for the button (pushed state)  - use NULL to
-	disable drawing of image	
-*************************************************************************/
-void WLButton::setPushedImage(const RenderableImage* image)
-{
-	if (image == NULL)
-	{
-		d_usePushedImage = false;
-	}
-	else
-	{
-		d_usePushedImage = true;
-		d_pushedImage = *image;
-		d_pushedImage.setRect(Rect(0, 0, d_abs_area.getWidth(), d_abs_area.getHeight()));
-	}
-
-	requestRedraw();
-}
-
-
-/*************************************************************************
-	set the image to render for the button (disabled state)  - use NULL
-	to disable drawing of image	
-*************************************************************************/
-void WLButton::setDisabledImage(const RenderableImage* image)
-{
-	if (image == NULL)
-	{
-		d_useDisabledImage = false;
-	}
-	else
-	{
-		d_useDisabledImage = true;
-		d_disabledImage = *image;
-		d_disabledImage.setRect(Rect(0, 0, d_abs_area.getWidth(), d_abs_area.getHeight()));
-	}
-
-	requestRedraw();
-}
-
-
 /*************************************************************************
 	render Widget in normal state	
 *************************************************************************/
@@ -296,8 +189,9 @@ void WLButton::drawNormal(float z)
 	// Draw label text
 	//
 	colours.setColours(d_normalColour);
-	colours.setAlpha(alpha_comp);
+	colours.modulateAlpha(alpha_comp);
 	absrect.d_top += PixelAligned((absrect.getHeight() - getFont()->getLineSpacing()) / 2);
+    absrect.d_left += PixelAligned(d_textXOffset * absrect.getWidth());
 	getFont()->drawText(getText(), absrect, System::getSingleton().getRenderer()->getZLayer(2), clipper, Centred, colours);
 }
 
@@ -352,8 +246,9 @@ void WLButton::drawHover(float z)
 	// Draw label text
 	//
 	colours.setColours(d_hoverColour);
-	colours.setAlpha(alpha_comp);
+	colours.modulateAlpha(alpha_comp);
 	absrect.d_top += PixelAligned((absrect.getHeight() - getFont()->getLineSpacing()) / 2);
+    absrect.d_left += PixelAligned(d_textXOffset * absrect.getWidth());
 	getFont()->drawText(getText(), absrect, System::getSingleton().getRenderer()->getZLayer(2), clipper, Centred, colours);
 }
 
@@ -408,8 +303,9 @@ void WLButton::drawPushed(float z)
 	// Draw label text
 	//
 	colours.setColours(d_pushedColour);
-	colours.setAlpha(alpha_comp);
+	colours.modulateAlpha(alpha_comp);
 	absrect.d_top += PixelAligned((absrect.getHeight() - getFont()->getLineSpacing()) / 2);
+    absrect.d_left += PixelAligned(d_textXOffset * absrect.getWidth());
 	getFont()->drawText(getText(), absrect, System::getSingleton().getRenderer()->getZLayer(2), clipper, Centred, colours);
 }
 
@@ -464,35 +360,13 @@ void WLButton::drawDisabled(float z)
 	// Draw label text
 	//
 	colours.setColours(d_disabledColour);
-	colours.setAlpha(alpha_comp);
+	colours.modulateAlpha(alpha_comp);
 	absrect.d_top += PixelAligned((absrect.getHeight() - getFont()->getLineSpacing()) / 2);
+    absrect.d_left += PixelAligned(d_textXOffset * absrect.getWidth());
 	getFont()->drawText(getText(), absrect, System::getSingleton().getRenderer()->getZLayer(2), clipper, Centred, colours);
 }
 
 
-/*************************************************************************
-	Set whether to auto re-size custom image areas when the button is
-	sized.	
-*************************************************************************/
-void WLButton::setCustomImageryAutoSized(bool setting)
-{
-	// if we are enabling auto-sizing, scale images for current size
-	if ((setting == true) && (setting != d_autoscaleImages))
-	{
-		Rect area(0, 0, d_abs_area.getWidth(), d_abs_area.getHeight());
-
-		d_normalImage.setRect(area);
-		d_hoverImage.setRect(area);
-		d_pushedImage.setRect(area);
-		d_disabledImage.setRect(area);
-
-		requestRedraw();
-	}
-
-	d_autoscaleImages = setting;
-}
-
-
 /*************************************************************************
 	Store sizes of frame edges
 *************************************************************************/
@@ -522,17 +396,6 @@ void WLButton::onSized(WindowEventArgs& e)
 	d_hoverFrame.setSize(absSize);
 	d_pushedFrame.setSize(absSize);
 
-	// scale user images if required.
-	if (d_autoscaleImages)
-	{
-		Rect area(0, 0, absSize.d_width, absSize.d_height);
-
-		d_normalImage.setRect(area);
-		d_hoverImage.setRect(area);
-		d_pushedImage.setRect(area);
-		d_disabledImage.setRect(area);
-	}
-
 	e.handled = true;
 }
 
@@ -574,10 +437,7 @@ void WLButton::onAlphaChanged(WindowEventArgs& e)
 *************************************************************************/
 Window* WLButtonFactory::createWindow(const String& name)
 {
-	WLButton* wnd = new WLButton(d_type, name);
-	wnd->initialise();
-
-	return wnd;
+	return new WLButton(d_type, name);
 }
 
 
