@@ -29,6 +29,8 @@
 #include "WLTitlebar.h"
 #include "WLCloseButton.h"
 
+#include <math.h>
+
 // Start of CEGUI namespace section
 namespace CEGUI
 {
@@ -70,12 +72,12 @@ const float	WLFrameWindow::TitlebarYOffset			= 0;
 const float	WLFrameWindow::TitlebarTextPadding		= 12;
 
 // colours
-const colour WLFrameWindow::ActiveColour			= 0xA7C7FF;
-const colour WLFrameWindow::InactiveColour			= 0xEFEFEF;
-const colour WLFrameWindow::ClientTopLeftColour		= 0xDFDFF5;
-const colour WLFrameWindow::ClientTopRightColour	= 0xDFEFF5;
-const colour WLFrameWindow::ClientBottomLeftColour	= 0xF4F3F5;
-const colour WLFrameWindow::ClientBottomRightColour	= 0xF0F0F5;
+const colour WLFrameWindow::ActiveColour			= 0xFFA7C7FF;
+const colour WLFrameWindow::InactiveColour			= 0xFFEFEFEF;
+const colour WLFrameWindow::ClientTopLeftColour		= 0xFFDFDFF5;
+const colour WLFrameWindow::ClientTopRightColour	= 0xFFDFEFF5;
+const colour WLFrameWindow::ClientBottomLeftColour	= 0xFFF4F3F5;
+const colour WLFrameWindow::ClientBottomRightColour	= 0xFFF0F0F5;
 
 
 /*************************************************************************
@@ -138,6 +140,9 @@ WLFrameWindow::~WLFrameWindow(void)
 *************************************************************************/
 Rect WLFrameWindow::getUnclippedInnerRect(void) const
 {
+    if (d_rolledup)
+        return Rect(0,0,0,0);
+
 	Rect tmp(getUnclippedPixelRect());
 
 	if (isFrameEnabled())
@@ -164,9 +169,9 @@ Rect WLFrameWindow::getUnclippedInnerRect(void) const
 	Create a control based upon the Titlebar base class to be used as
 	the title bar for this window.
 *************************************************************************/
-Titlebar* WLFrameWindow::createTitlebar(void) const
+Titlebar* WLFrameWindow::createTitlebar(const String& name) const
 {
-	Titlebar* tbar = (Titlebar*)WindowManager::getSingleton().createWindow(TitlebarType, getName() + "__auto_titlebar__");
+	Titlebar* tbar = (Titlebar*)WindowManager::getSingleton().createWindow(TitlebarType, name);
 	tbar->setMetricsMode(Absolute);
 	tbar->setPosition(Point(TitlebarXOffset, TitlebarYOffset));
 
@@ -178,9 +183,9 @@ Titlebar* WLFrameWindow::createTitlebar(void) const
 	Create a control based upon the PushButton base class, to be used as
 	the close button for the window.
 *************************************************************************/
-PushButton* WLFrameWindow::createCloseButton(void) const
+PushButton* WLFrameWindow::createCloseButton(const String& name) const
 {
-	WLButton* btn = (WLButton*)WindowManager::getSingleton().createWindow(CloseButtonType, getName() + "__auto_closebutton__");
+	WLButton* btn = (WLButton*)WindowManager::getSingleton().createWindow(CloseButtonType, name);
 
 	btn->setStandardImageryEnabled(false);
 	btn->setCustomImageryAutoSized(true);
@@ -213,24 +218,25 @@ PushButton* WLFrameWindow::createCloseButton(void) const
 	Setup size and position for the title bar and close button widgets
 	attached to this window
 *************************************************************************/
-void WLFrameWindow::layoutComponentWidgets()
+void WLFrameWindow::performChildWindowLayout()
 {
-	// TODO: Magic number removal
+    FrameWindow::performChildWindowLayout();
 
 	ImagesetManager& ismgr = ImagesetManager::getSingleton();
 
 	// calculate and set size of title bar
 	Size titleSz;
 	titleSz.d_height = d_titlebar->getFont()->getLineSpacing() + TitlebarTextPadding;
-	titleSz.d_width	 = isRolledup() ? d_abs_openSize.d_width : d_abs_area.getWidth();
+	titleSz.d_width	 = getAbsoluteWidth();
 	d_titlebar->setSize(titleSz);
 
-	// set size of close button to be the same as the height for the title bar.
-	float closeSize = titleSz.d_height * 0.75f;
+	// set size of close button to be the same as the size of the imagery used to render it.
+	float closeSize = ismgr.getImageset(ImagesetName)->getImage(CloseButtonNormalImageName).getWidth();
 	d_closeButton->setSize(Size(closeSize, closeSize));
 
 	// calculate position for close button (somewhere over the end of the titlbar)
-	float closeX = titleSz.d_width - closeSize - titleSz.d_height * 0.125f;
+	float closeX = titleSz.d_width - closeSize - 
+		ismgr.getImageset(WLTitlebar::ImagesetName)->getImage(WLTitlebar::RightFrameImageName).getWidth();
 
 	float closeY = TitlebarYOffset + ((titleSz.d_height - closeSize) / 2);
 
@@ -243,6 +249,10 @@ void WLFrameWindow::layoutComponentWidgets()
 *************************************************************************/
 void WLFrameWindow::drawSelf(float z)
 {
+    // do nothing when rolled up.
+    if (d_rolledup)
+        return;
+    
 	// get the destination screen rect for this window
 	Rect absrect(getUnclippedPixelRect());
 
@@ -292,11 +302,7 @@ void WLFrameWindow::onSized(WindowEventArgs& e)
 	newsz.d_height -= frame_offset;
 	d_frame.setSize(newsz);
 
-	// adjust position for client brush
-	pos.d_y += (isTitleBarEnabled() || !isFrameEnabled()) ? 0 : d_frameTopSize;
-
-	// modify size of client so it is within the frame
-	if (isFrameEnabled())
+	// adjust position and size of client so it is within the frame	if (isFrameEnabled())
 	{
 		pos.d_x += d_frameLeftSize;
 		newsz.d_width	-= (d_frameLeftSize + d_frameRightSize);
@@ -304,6 +310,7 @@ void WLFrameWindow::onSized(WindowEventArgs& e)
 
 		if (!isTitleBarEnabled())
 		{
+			pos.d_y += d_frameTopSize;
 			newsz.d_height -= d_frameTopSize;
 		}
 	}
@@ -334,6 +341,25 @@ void WLFrameWindow::onAlphaChanged(WindowEventArgs& e)
 }
 
 
+/*************************************************************************
+    Handler for when clip mode changes
+*************************************************************************/
+void WLFrameWindow::onClippingChanged(WindowEventArgs& e)
+{
+    FrameWindow::onClippingChanged(e);
+
+    // set same mode on the titlebar and close button
+    if (d_titlebar)
+    {
+        d_titlebar->setClippedByParent(d_clippedByParent);
+    }
+    if (d_closeButton)
+    {
+        d_closeButton->setClippedByParent(d_clippedByParent);
+    }
+}
+
+
 /*************************************************************************
 	Handler for when window is activated
 *************************************************************************/
@@ -365,16 +391,16 @@ void WLFrameWindow::storeFrameSizes(void)
 
 	const Image* img;
 	img = &iset->getImage(LeftFrameImageName);
-	d_frameLeftSize = img->getWidth() + img->getOffsetX();
+	d_frameLeftSize = img->getWidth() + fabsf(img->getOffsetX());
 
-	img = &iset->getImage(LeftFrameImageName);
-	d_frameRightSize = img->getWidth() + img->getOffsetX();
+	img = &iset->getImage(RightFrameImageName);
+	d_frameRightSize = img->getWidth() + fabsf(img->getOffsetX());
 
 	img = &iset->getImage(TopFrameImageName);
-	d_frameTopSize = img->getHeight() + img->getOffsetY();
+	d_frameTopSize = img->getHeight() + fabsf(img->getOffsetY());
 
 	img = &iset->getImage(BottomFrameImageName);
-	d_frameBottomSize = img->getHeight() + img->getOffsetY();
+	d_frameBottomSize = img->getHeight() + fabsf(img->getOffsetY());
 }
 
 
@@ -483,10 +509,7 @@ void WLFrameWindow::updateFrameColours(void)
 *************************************************************************/
 Window* WLFrameWindowFactory::createWindow(const String& name)
 {
-	WLFrameWindow* wnd = new WLFrameWindow(d_type, name);
-	wnd->initialise();
-
-	return wnd;
+	return new WLFrameWindow(d_type, name);
 }
 
 } // End of  CEGUI namespace section
