@@ -55,7 +55,7 @@ namespace CEGUI
 	of 'components' which can later be accessed via name.  The components of an Imageset can queried for
 	various details, and sent to the Renderer object for drawing.
 */
-class CEGUIBASE_API Imageset
+class CEGUIEXPORT Imageset
 {
 	friend class Imageset_xmlHandler;
 private:
@@ -66,6 +66,7 @@ private:
 	*************************************************************************/
 	friend Imageset*	ImagesetManager::createImageset(const String& name, Texture* texture);
 	friend Imageset*	ImagesetManager::createImageset(const String& filename, const String& resourceGroup);
+	friend Imageset*	ImagesetManager::createImagesetFromImageFile(const String& name, const String& filename, const String& resourceGroup);
 	friend void			ImagesetManager::destroyImageset(const String& name);
 
 
@@ -99,6 +100,34 @@ private:
 	Imageset(const String& filename, const String& resourceGroup);
 
 
+    /*!
+    \brief
+        Construct a new Imageset using the specified image file and imageset name.  The created
+        imageset will, by default, have a single Image defined named "full_image" which represents
+        the entire area of the loaded image file.
+
+    \note
+        Under certain renderers it may be required that the source image dimensions be some
+        power of 2, if this condition is not met then stretching and other undesired side-effects
+        may be experienced.  To be safe from such effects it is generally recommended that all
+        images that you load have dimensions that are some power of 2.
+
+    \param name
+        String object holding the name to be assigned to the created imageset.
+
+    \param filename
+        String object holding the filename of the image that is to be loaded.  The image should be
+        of some format that is supported by the Renderer that is in use.
+
+    \param resourceGroup
+        Resource group identifier to be passed to the resource manager, which may specify a group
+        from which the image file is to be loaded.
+
+    \exception FileIOException thrown if something goes wrong while loading the image.
+    */
+    Imageset(const String& name, const String& filename, const String& resourceGroup);
+
+
 public:	// For luabind support
 	/*!
 	\brief
@@ -393,7 +422,7 @@ public:
 	\return
 		Nothing
 	*/
-	void	draw(const Rect& source_rect, const Rect& dest_rect, float z, const Rect& clip_rect, const colour& top_left_colour = -1, const colour& top_right_colour = -1,  const colour& bottom_left_colour = -1, const colour& bottom_right_colour = -1, QuadSplitMode quad_split_mode = TopLeftToBottomRight) const
+	void	draw(const Rect& source_rect, const Rect& dest_rect, float z, const Rect& clip_rect, const colour& top_left_colour = 0xFFFFFFFF, const colour& top_right_colour = 0xFFFFFFFF,  const colour& bottom_left_colour = 0xFFFFFFFF, const colour& bottom_right_colour = 0xFFFFFFFF, QuadSplitMode quad_split_mode = TopLeftToBottomRight) const
 	{
 		draw(source_rect, dest_rect, z, clip_rect, ColourRect(top_left_colour, top_right_colour, bottom_left_colour, bottom_right_colour), quad_split_mode);
 	}
@@ -465,6 +494,19 @@ public:
 	ImageIterator	getIterator(void) const;
 
 
+    /*!
+    \brief
+        Writes an xml representation of this Imageset to \a out_stream.
+
+    \param out_stream
+        Stream where xml data should be output.
+
+    \return
+        Nothing.
+    */
+    void writeXMLToStream(OutStream& out_stream) const;
+
+
 protected:
 	/*************************************************************************
 		Implementation Constants
@@ -531,6 +573,7 @@ protected:
 	String			d_name;						//!< Holds the name of this imageset.
 	ImageRegistry	d_images;					//!< Registry of Image objects for the images defined for this Imageset
 	Texture*		d_texture;					//!< Texture object that handles imagery for this Imageset
+    String          d_textureFilename;          //!< String holding the name of the texture filename (if any).
 
 	// auto-scaling fields
 	bool	d_autoScale;			//!< true when auto-scaling is enabled.
