@@ -2,7 +2,7 @@
 	filename: 	CEGUISystem.h
 	created:	20/2/2004
 	author:		Paul D Turner
-	
+
 	purpose:	Defines interface for main GUI system class
 *************************************************************************/
 /*************************************************************************
@@ -45,7 +45,7 @@
 // Start of CEGUI namespace section
 namespace CEGUI
 {
-// forward reference of impl struct
+//! Implementation struct that tracks and controls multiclick for mouse buttons.
 struct MouseClickTrackerImpl;
 
 
@@ -57,7 +57,7 @@ struct MouseClickTrackerImpl;
 	an initialised Renderer object which it can use to interface to whatever rendering system will be
 	used to display the GUI imagery.
 */
-class CEGUIBASE_API System : public Singleton<System>, public EventSet
+class CEGUIEXPORT System : public Singleton<System>, public EventSet
 {
 public:
 	static const String EventNamespace;				//!< Namespace for global events
@@ -92,7 +92,7 @@ public:
 	\param logFile
 		pointer to a utf8 encoded buffer containing the name to use for the log file.
 	*/
-	System(Renderer* renderer, utf8* logFile = (utf8*)"CEGUI.log");
+	System(Renderer* renderer, const utf8* logFile = (const utf8*)"CEGUI.log");
 
 	/*!
 	\brief
@@ -107,7 +107,7 @@ public:
 	\param logFile
 		pointer to a utf8 encoded buffer containing the name to use for the log file.
 	*/
-	System(Renderer* renderer, ResourceProvider* resourceProvider, utf8* logFile = (utf8*)"CEGUI.log");
+    System(Renderer* renderer, ResourceProvider* resourceProvider, const utf8* logFile = (const utf8*)"CEGUI.log");
 
 
 	/*!
@@ -123,7 +123,7 @@ public:
 	\param configFile
 		pointer to a utf8 encoded buffer containing the name to use for the configuration file.
 	*/
-	System(Renderer* renderer, ScriptModule* scriptModule, utf8* configFile = (utf8*)"cegui.config");
+	System(Renderer* renderer, ScriptModule* scriptModule, const utf8* configFile = (const utf8*)"cegui.config");
 
 
 	/*!
@@ -142,7 +142,83 @@ public:
 	\param configFile
 		pointer to a utf8 encoded buffer containing the name to use for the configuration file.
 	*/
-	System(Renderer* renderer, ScriptModule* scriptModule, ResourceProvider* resourceProvider, utf8* configFile = (utf8*)"cegui.config");
+	System(Renderer* renderer, ScriptModule* scriptModule, ResourceProvider* resourceProvider, const utf8* configFile = (const utf8*)"cegui.config");
+
+
+    /*!
+    \brief
+        Constructs a System object.
+
+    \param renderer
+        Pointer to the valid Renderer object that will be used to render GUI imagery
+
+    \param xmlParser
+        Pointer to a valid XMLParser object to be used when parsing XML files, or NULL to use a default parser.
+
+    \param logFile
+        pointer to a utf8 encoded buffer containing the name to use for the log file.
+    */
+    System(Renderer* renderer, XMLParser* xmlParser, const utf8* logFile = (const utf8*)"CEGUI.log");
+
+    
+    /*!
+    \brief
+        Constructs a System object.
+
+    \param renderer
+        Pointer to the valid Renderer object that will be used to render GUI imagery
+
+    \param resourceProvider
+        Pointer to a ResourceProvider object.
+    
+    \param xmlParser
+        Pointer to a valid XMLParser object to be used when parsing XML files, or NULL to use a default parser.
+
+    \param logFile
+        pointer to a utf8 encoded buffer containing the name to use for the log file.
+    */
+    System(Renderer* renderer, ResourceProvider* resourceProvider, XMLParser* xmlParser, const utf8* logFile = (const utf8*)"CEGUI.log");
+
+    
+    /*!
+    \brief
+        Construct a new System object
+
+    \param renderer
+        Pointer to the valid Renderer object that will be used to render GUI imagery
+
+    \param xmlParser
+        Pointer to a valid XMLParser object to be used when parsing XML files, or NULL to use a default parser.
+    
+    \param scriptModule
+        Pointer to a ScriptModule object.  may be NULL.
+
+    \param configFile
+        pointer to a utf8 encoded buffer containing the name to use for the configuration file.
+    */
+    System(Renderer* renderer, XMLParser* xmlParser, ScriptModule* scriptModule, const utf8* configFile = (const utf8*)"cegui.config");
+
+        
+    /*!
+    \brief
+        Construct a new System object
+
+    \param renderer
+        Pointer to the valid Renderer object that will be used to render GUI imagery
+
+    \param resourceProvider
+        Pointer to a ResourceProvider object.
+    
+    \param xmlParser
+        Pointer to a valid XMLParser object to be used when parsing XML files, or NULL to use a default parser.
+    
+    \param scriptModule
+        Pointer to a ScriptModule object.  may be NULL.
+
+    \param configFile
+        pointer to a utf8 encoded buffer containing the name to use for the configuration file.
+    */
+    System(Renderer* renderer, ResourceProvider* resourceProvider, XMLParser* xmlParser, ScriptModule* scriptModule, const utf8* configFile = (const utf8*)"cegui.config");
 
 
 	/*!
@@ -228,6 +304,16 @@ public:
 	void	signalRedraw()		{d_gui_redraw = true;}
 
 
+	/*!
+	\brief
+		Return a boolean value to indicate whether a full re-draw is requested next time renderGUI() is called.
+
+	\return
+		true if a re-draw has been requested
+	*/
+	bool	isRedrawRequested() const		{return d_gui_redraw;}
+
+
 	/*!
 	\brief
 		Render the GUI
@@ -440,8 +526,11 @@ public:
 
 	\param filename
 		String object holding the filename of the script file that is to be executed
+		
+	\param resourceGroup
+		Resource group identifier to be passed to the ResourceProvider when loading the script file.
 	*/
-	void	executeScriptFile(const String& filename) const;
+	void	executeScriptFile(const String& filename, const String& resourceGroup = "") const;
 
 
 	/*!
@@ -455,10 +544,23 @@ public:
 	\return
 		The integer value returned from the script function.
 	*/
-	int		executeScriptGloabl(const String& function_name) const;
+	int		executeScriptGlobal(const String& function_name) const;
 
 
-	/*!
+    /*!
+    \brief
+        If possible, execute script code contained in the given CEGUI::String object.
+
+    \param str
+        String object holding the valid script code that should be executed.
+
+    \return
+        Nothing.
+    */
+    void executeScriptString(const String& str) const;
+
+
+    /*!
 	\brief
 		return the current mouse movement scaling factor.
 
@@ -501,6 +603,75 @@ public:
     */
     uint    getSystemKeys(void) const   { return d_sysKeys; }
 
+    /*!
+    \brief
+        Return the XMLParser object.
+     */
+    XMLParser* getXMLParser(void) const     { return d_xmlParser; }
+
+
+    /*!
+    \brief
+        Set the system default Tooltip object.  This value may be NULL to indicate that no default Tooltip will be
+        available.
+
+    \param tooltip
+        Pointer to a valid Tooltip based object which should be used as the default tooltip for the system, or NULL to
+        indicate that no system default tooltip is required.  Note that when passing a pointer to a Tooltip object,
+        ownership of the Tooltip does not pass to System.
+
+    \return
+        Nothing.
+    */
+    void setTooltip(Tooltip* tooltip);
+
+    /*!
+    \brief
+        Set the system default Tooltip to be used by specifying a Window type.
+
+        System will internally attempt to create an instance of the specified window type (which must be
+        derived from the base Tooltip class).  If the Tooltip creation fails, the error is logged and no
+        system default Tooltip will be available.
+
+    \param tooltipType
+        String object holding the name of the Tooltip based Window type which should be used as the Tooltip for
+        the system default.
+
+    \return
+        Nothing.
+    */
+    void setTooltip(const String& tooltipType);
+
+    /*!
+    \brief
+        return a poiter to the system default tooltip.  May return 0.
+
+    \return
+        Pointer to the current system default tooltip.  May return 0 if
+        no system default tooltip is available.
+     */
+    Tooltip* getDefaultTooltip(void) const  { return d_defaultTooltip; }
+
+
+	/*!
+	\brief
+		Internal method to directly set the current modal target.
+
+	\note
+		This method is called internally by Window, and must be used by client code.
+		Doing so will most likely not have the expected results.
+	*/
+	void setModalTarget(Window* target)		{d_modalTarget = target;}
+
+	/*!
+	\brief
+		Return a pointer to the Window that is currently the modal target.
+
+	\return
+		Pointer to the current modal target. NULL if there is no modal target.
+	*/
+	Window* getModalTarget(void) const		{return d_modalTarget;}
+
 
 	/*************************************************************************
 		Input injection interface
@@ -522,6 +693,17 @@ public:
 	bool	injectMouseMove(float delta_x, float delta_y);
 
 
+	/*!
+	\brief
+		Method that injects that the mouse has left the application window
+
+	\return
+		- true if the generated mouse move event was handled.
+		- false if the generated mouse move event was not handled.
+	*/
+	bool	injectMouseLeaves(void);
+
+
 	/*!
 	\brief
 		Method that injects a mouse button down event into the system.
@@ -637,6 +819,10 @@ public:
 
 
 private:
+    // unimplemented constructors / assignment
+    System(const System& obj);
+    System& operator=(const System& obj);
+
 	/*************************************************************************
 		Implementation Constants
 	*************************************************************************/
@@ -659,6 +845,29 @@ private:
 	Window*	getTargetWindow(const Point& pt) const;
 
 
+	/*!
+	\brief
+		Return a pointer to the Window that should receive keyboard input considering the current modal target.
+
+	\return
+		Pointer to a Window object that should receive the keyboard input.
+	*/
+	Window* getKeyboardTargetWindow(void) const;
+
+
+	/*!
+	\brief
+		Return a pointer to the next window that is to receive the input if the given Window did not use it.
+
+	\param w
+		Pointer to the Window that just received the input.
+
+	\return
+		Pointer to the next window to receive the input.
+	*/
+	Window* getNextTargetWindow(Window* w) const;
+
+
 	/*!
 	\brief
 		Translate a MouseButton value into the corresponding SystemKey value
@@ -675,7 +884,7 @@ private:
 	/*!
 	\brief
 		Translate a Key::Scan value into the corresponding SystemKey value.
-		
+
 		This takes key direction into account, since we map two keys onto one value.
 
 	\param key
@@ -694,7 +903,7 @@ private:
 	\brief
 		Method to do the work of the constructor
 	*/
-	void	constructor_impl(Renderer* renderer, ResourceProvider* resourceProvider, ScriptModule* scriptModule, const String& configFile, const String& logFile);
+       void	constructor_impl(Renderer* renderer, ResourceProvider* resourceProvider, XMLParser* xmlParser, ScriptModule* scriptModule, const String& configFile, const String& logFile);
 
 
 	/*!
@@ -775,12 +984,17 @@ private:
 
 	Window*		d_wndWithMouse;		//!< Pointer to the window that currently contains the mouse.
 	Window*		d_activeSheet;		//!< The active GUI sheet (root window)
+	Window*		d_modalTarget;		//!< Pointer to the window that is the current modal target. NULL is there is no modal target.
+
+	String d_strVersion;    //!< CEGUI version
 
 	uint		d_sysKeys;			//!< Current set of system keys pressed (in mk1 these were passed in, here we track these ourself).
 	bool		d_lshift;			//!< Tracks state of left shift.
 	bool		d_rshift;			//!< Tracks state of right shift.
 	bool		d_lctrl;			//!< Tracks state of left control.
 	bool		d_rctrl;			//!< Tracks state of right control.
+	bool		d_lalt;				//!< Tracks state of left alt.
+	bool		d_ralt;				//!< Tracks state of right alt.
 
 	double		d_click_timeout;	//!< Timeout value, in seconds, used to generate a single-click (button down then up)
 	double		d_dblclick_timeout;	//!< Timeout value, in seconds, used to generate multi-click events (botton down, then up, then down, and so on).
@@ -796,6 +1010,12 @@ private:
 	String			d_termScriptName;		//!< Name of the script to run upon system shutdown.
 
 	float	d_mouseScalingFactor;			//!< Scaling applied to mouse movement inputs.
+
+    XMLParser*  d_xmlParser;        //!< XMLParser object we use to process xml files.
+    bool        d_ourXmlParser;     //!< true when we created the xml parser.
+
+    Tooltip* d_defaultTooltip;      //!< System default tooltip object.
+    bool     d_weOwnTooltip;        //!< true if System created the custom Tooltip.
 };
 
 } // End of  CEGUI namespace section
