@@ -60,7 +60,7 @@ public:
 	RelativeMinSize() : Property(
 		"RelativeMinSize",
 		"Property to get/set the minimum size for the Window.  Value is \"w:[float] h:[float]\" using relative metrics (this setting is relative to the display size).",
-		"w:0.000000 h:0.000000")
+		"w:0.000000 h:0.000000", false)
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -88,7 +88,7 @@ public:
 	RelativeMaxSize() : Property(
 		"RelativeMaxSize",
 		"Property to get/set the maximum size for the Window.  Value is \"w:[float] h:[float]\" using relative metrics (this setting is relative to the display size).",
-		"w:1.000000 h:1.000000")
+		"w:1.000000 h:1.000000", false)
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -116,7 +116,7 @@ public:
 	AbsoluteMinSize() : Property(
 		"AbsoluteMinSize",
 		"Property to get/set the minimum size for the Window.  Value is \"w:[float] h:[float]\" using absolute (pixel) metrics.",
-		"w:0.000000 h:0.000000")
+		"w:0.000000 h:0.000000", false)
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -144,7 +144,7 @@ public:
 	AbsoluteMaxSize() : Property(
 		"AbsoluteMaxSize",
 		"Property to get/set the maximum size for the Window.  Value is \"w:[float] h:[float]\" using absolute (pixel) metrics.",
-		"")
+		"", false)
 		{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -263,6 +263,7 @@ public:
 
 	String	get(const PropertyReceiver* receiver) const;
 	void	set(PropertyReceiver* receiver, const String& value);
+	bool	isDefault(const PropertyReceiver* receiver) const;
 };
 
 
@@ -318,6 +319,7 @@ public:
 
 	String	get(const PropertyReceiver* receiver) const;
 	void	set(PropertyReceiver* receiver, const String& value);
+	bool	isDefault(const PropertyReceiver* receiver) const;
 };
 
 
@@ -430,6 +432,7 @@ public:
 
 	String	get(const PropertyReceiver* receiver) const;
 	void	set(PropertyReceiver* receiver, const String& value);
+	bool	isDefault(const PropertyReceiver* receiver) const;
 };
 
 
@@ -458,6 +461,7 @@ public:
 
 	String	get(const PropertyReceiver* receiver) const;
 	void	set(PropertyReceiver* receiver, const String& value);
+	bool	isDefault(const PropertyReceiver* receiver) const;
 };
 
 
@@ -537,7 +541,7 @@ public:
 	Width() : Property(
 		"Width",
 		"Property to get/set the width of the Window.  Value is floating point using the active metrics mode.",
-		"0.000000") {}
+		"0.000000", false) {}
 
 	String	get(const PropertyReceiver* receiver) const;
 	void	set(PropertyReceiver* receiver, const String& value);
@@ -563,7 +567,7 @@ public:
 	RelativeWidth() : Property(
 		"RelativeWidth",
 		"Property to get/set the width of the Window.  Value is floating point using relative metrics.",
-		"0.000000")
+		"0.000000", false)
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -590,7 +594,7 @@ public:
 	AbsoluteWidth() : Property(
 		"AbsoluteWidth",
 		"Property to get/set the width of the Window.  Value is floating point using absolute metrics.",
-		"0.000000")
+		"0.000000", false)
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -617,7 +621,7 @@ public:
 	Height() : Property(
 		"Height",
 		"Property to get/set the height of the Window.  Value is floating point using the active metrics mode.",
-		"0.000000")
+		"0.000000", false)
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -644,7 +648,7 @@ public:
 	RelativeHeight() : Property(
 		"RelativeHeight",
 		"Property to get/set the height of the Window.  Value is floating point using relative metrics.",
-		"0.000000")
+		"0.000000", false)
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -671,7 +675,7 @@ public:
 	AbsoluteHeight() : Property(
 		"AbsoluteHeight",
 		"Property to get/set the height of the Window.  Value is floating point using absolute metrics.",
-		"0.000000")
+		"0.000000", false)
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -699,7 +703,7 @@ public:
 	Size() : Property(
 		"Size",
 		"Property to get/set the size of the Window.  Value is \"w:[float] h:[float]\" using the active metrics mode.",
-		"w:0.000000 h:0.000000")
+		"w:0.000000 h:0.000000", false)
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -727,7 +731,7 @@ public:
 	RelativeSize() : Property(
 		"RelativeSize",
 		"Property to get/set the size of the Window.  Value is \"w:[float] h:[float]\" using relative metrics.",
-		"w:0.000000 h:0.000000")
+		"w:0.000000 h:0.000000", false)
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -755,7 +759,7 @@ public:
 	AbsoluteSize() : Property(
 		"AbsoluteSize",
 		"Property to get/set the size of the Window.  Value is \"w:[float] h:[float]\" using absolute metrics.",
-		"w:0.000000 h:0.000000")
+		"w:0.000000 h:0.000000", false)
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -782,7 +786,7 @@ public:
 	XPosition() : Property(
 		"XPosition",
 		"Property to get/set the x co-ordinate position of the Window.  Value is a floating point number using the active metrics mode.",
-		"0.000000")
+		"0.000000", false)
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -809,7 +813,7 @@ public:
 	RelativeXPosition() : Property(
 		"RelativeXPosition",
 		"Property to get/set the x co-ordinate position of the Window.  Value is a floating point number using relative metrics.",
-		"0.000000")
+		"0.000000", false)
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -836,7 +840,7 @@ public:
 	AbsoluteXPosition() : Property(
 		"AbsoluteXPosition",
 		"Property to get/set the x co-ordinate position of the Window.  Value is a floating point number using absolute metrics.",
-		"0.000000")
+		"0.000000", false)
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -863,7 +867,7 @@ public:
 	YPosition() : Property(
 		"YPosition",
 		"Property to get/set the y co-ordinate position of the Window.  Value is a floating point number using the active metrics mode.",
-		"0.000000")
+		"0.000000", false)
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -890,7 +894,7 @@ public:
 	RelativeYPosition() : Property(
 		"RelativeYPosition",
 		"Property to get/set the y co-ordinate position of the Window.  Value is a floating point number using relative metrics.",
-		"0.000000")
+		"0.000000", false)
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -917,7 +921,7 @@ public:
 	AbsoluteYPosition() : Property(
 		"AbsoluteYPosition",
 		"Property to get/set the y co-ordinate position of the Window.  Value is a floating point number using absolute metrics.",
-		"0.000000")
+		"0.000000", false)
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -945,7 +949,7 @@ public:
 	Position() : Property(
 		"Position",
 		"Property to get/set the position of the Window.  Value is \"x:[float] y:[float]\" using the active metrics mode.",
-		"x:0.000000 y:0.000000")
+		"x:0.000000 y:0.000000", false)
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -973,7 +977,7 @@ public:
 	RelativePosition() : Property(
 		"RelativePosition",
 		"Property to get/set the position of the Window.  Value is \"x:[float] y:[float]\" using relative metrics.",
-		"x:0.000000 y:0.000000")
+		"x:0.000000 y:0.000000", false)
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -1001,7 +1005,7 @@ public:
 	AbsolutePosition() : Property(
 		"AbsolutePosition",
 		"Property to get/set the position of the Window.  Value is \"x:[float] y:[float]\" using absolute metrics.",
-		"x:0.000000 y:0.000000")
+		"x:0.000000 y:0.000000", false)
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -1031,7 +1035,7 @@ public:
 	Rect() : Property(
 		"Rect",
 		"Property to get/set the area rectangle of the Window.  Value is \"l:[float] t:[float] r:[float] b:[float]\" (where l is left, t is top, r is right, and b is bottom) using the active metrics system.",
-		"l:0.000000 t:0.000000 r:0.000000 b:0.000000")
+		"l:0.000000 t:0.000000 r:0.000000 b:0.000000", false)
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -1061,7 +1065,7 @@ public:
 	RelativeRect() : Property(
 		"RelativeRect",
 		"Property to get/set the area rectangle of the Window.  Value is \"l:[float] t:[float] r:[float] b:[float]\" (where l is left, t is top, r is right, and b is bottom) using relative metrics.",
-		"l:0.000000 t:0.000000 r:0.000000 b:0.000000") 
+		"l:0.000000 t:0.000000 r:0.000000 b:0.000000", false) 
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -1091,7 +1095,7 @@ public:
 	AbsoluteRect() : Property(
 		"AbsoluteRect",
 		"Property to get/set the area rectangle of the Window.  Value is \"l:[float] t:[float] r:[float] b:[float]\" (where l is left, t is top, r is right, and b is bottom) using absolute metrics.",
-		"l:0.000000 t:0.000000 r:0.000000 b:0.000000") 
+		"l:0.000000 t:0.000000 r:0.000000 b:0.000000", false) 
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -1204,7 +1208,7 @@ public:
     AutoRepeatDelay() : Property(
         "AutoRepeatDelay",
         "Property to get/set the autorepeat delay.  Value is a floating point number indicating the delay required in seconds.",
-        "0.3")
+        "0.300000")
     {}
 
     String  get(const PropertyReceiver* receiver) const;
@@ -1231,13 +1235,440 @@ public:
     AutoRepeatRate() : Property(
         "AutoRepeatRate",
         "Property to get/set the autorepeat rate.  Value is a floating point number indicating the rate required in seconds.",
-        "0.06")
+        "0.060000")
     {}
 
     String  get(const PropertyReceiver* receiver) const;
     void    set(PropertyReceiver* receiver, const String& value);
 };
 
+/*!
+\brief
+	Property to access whether inputs are passed to child windows when
+    input is captured to this window.
+
+	\par Usage:
+		- Name: DistributeCapturedInputs
+		- Format: "[text]".
+
+	\par Where [Text] is:
+		- "True" to indicate 'captured' inputs should be passed to attached child windows.
+		- "False" to indicate 'captured' inputs should be passed to this window only.
+*/
+class DistributeCapturedInputs : public Property
+{
+public:
+	DistributeCapturedInputs() : Property(
+		"DistributeCapturedInputs",
+		"Property to get/set whether captured inputs are passed to child windows.  Value is either \"True\" or \"False\".",
+		"False")
+	{}
+
+	String	get(const PropertyReceiver* receiver) const;
+	void	set(PropertyReceiver* receiver, const String& value);
+};
+
+/*!
+\brief
+    Property to access the custom tooltip for this Window.
+
+    \par Usage:
+        - Name: CustomTooltipType
+        - Format: "[text]".
+
+    \par Where:
+        - [Text] is the typename of the custom tooltip for the Window.
+ */
+class CustomTooltipType : public Property
+{
+public:
+    CustomTooltipType() : Property(
+    "CustomTooltipType",
+    "Property to get/set the custom tooltip for the window.  Value is the type name of the custom tooltip.",
+    "")
+    {}
+
+    String  get(const PropertyReceiver* receiver) const;
+    void    set(PropertyReceiver* receiver, const String& value);
+};
+
+/*!
+\brief
+    Property to access the tooltip text for this Window.
+
+    \par Usage:
+        - Name: Tooltip
+        - Format: "[text]".
+
+    \par Where:
+        - [Text] is the tooltip text for this window.
+ */
+class Tooltip : public Property
+{
+public:
+    Tooltip() : Property(
+    "Tooltip",
+    "Property to get/set the tooltip text for the window.  Value is the tooltip text for the window.",
+    "")
+    {}
+
+    String  get(const PropertyReceiver* receiver) const;
+    void    set(PropertyReceiver* receiver, const String& value);
+};
+
+/*!
+\brief
+    Property to access whether the window inherits its tooltip text from its parent whn it has no tooltip text of its own.
+
+    \par Usage:
+        - Name: InheritsTooltipText
+        - Format: "[text]".
+
+    \par Where [Text] is:
+        - "True" to indicate the Window inherits its tooltip text from its parent.
+        - "False" to indicate the Window does not inherit its tooltip text.
+*/
+class InheritsTooltipText : public Property
+{
+public:
+    InheritsTooltipText() : Property(
+        "InheritsTooltipText",
+        "Property to get/set whether the window inherits its parents tooltip text when it has none of its own.  Value is either \"True\" or \"False\".",
+        "False")
+    {}
+
+    String  get(const PropertyReceiver* receiver) const;
+    void    set(PropertyReceiver* receiver, const String& value);
+};
+
+
+/*!
+\brief
+	Property to access whether the window rises to the top of the z order when clicked.
+
+	\par Usage:
+		- Name: RiseOnClick
+		- Format: "[text]".
+
+	\par Where [Text] is:
+		- "True" to indicate the Window will rise to the surface when clicked.
+		- "False" to indicate the Window will not change z position when clicked.
+*/
+class RiseOnClick : public Property
+{
+public:
+    RiseOnClick() : Property(
+		"RiseOnClick",
+		"Property to get/set whether the window will come tot he top of the z order hwn clicked.  Value is either \"True\" or \"False\".",
+		"True")
+	{}
+
+	String	get(const PropertyReceiver* receiver) const;
+	void	set(PropertyReceiver* receiver, const String& value);
+};
+
+
+/*!
+\brief
+    Property to access the vertical alignment setting for the window.
+
+    \par Usage:
+        - Name: VerticalAlignment
+        - Format: "[text]".
+
+    \par Where [Text] is:
+        - "Top" to indicate the windows position is an offset of its top edge from its parents top edge.
+        - "Centre" to indicate the windows position is an offset of its centre point from its parents centre point.
+        - "Bottom" to indicate the windows position is an offset of its bottom edge from its parents bottom edge.
+*/
+class VerticalAlignment : public Property
+{
+    public:
+        VerticalAlignment() : Property(
+        "VerticalAlignment",
+        "Property to get/set the windows vertical alignment.  Value is one of \"Top\", \"Centre\" or \"Bottom\".",
+        "Top")
+        {}
+
+        String	get(const PropertyReceiver* receiver) const;
+        void	set(PropertyReceiver* receiver, const String& value);
+};
+
+
+/*!
+\brief
+    Property to access the horizontal alignment setting for the window.
+
+    \par Usage:
+        - Name: HorizontalAlignment
+        - Format: "[text]".
+
+    \par Where [Text] is:
+        - "Left" to indicate the windows position is an offset of its left edge from its parents left edge.
+        - "Centre" to indicate the windows position is an offset of its centre point from its parents centre point.
+        - "Right" to indicate the windows position is an offset of its right edge from its parents right edge.
+*/
+class HorizontalAlignment : public Property
+{
+    public:
+        HorizontalAlignment() : Property(
+        "HorizontalAlignment",
+        "Property to get/set the windows horizontal alignment.  Value is one of \"Left\", \"Centre\" or \"Right\".",
+        "Left")
+        {}
+
+        String	get(const PropertyReceiver* receiver) const;
+        void	set(PropertyReceiver* receiver, const String& value);
+};
+
+
+/*
+\brief
+	Property to access the unified area rectangle of the window.
+
+	\par Usage:
+		- Name: UnifiedAreaRect
+		- Format: "{{[ls],[lo]},{[ts],[to]},{[rs],[ro]},{[bs],[bo]}}"
+
+	\par Where:
+		- [ls] is a floating point value describing the relative scale value for the left edge.
+		- [lo] is a floating point value describing the absolute offset value for the left edge.
+		- [ts] is a floating point value describing the relative scale value for the top edge.
+		- [to] is a floating point value describing the absolute offset value for the top edge.
+		- [rs] is a floating point value describing the relative scale value for the right edge.
+		- [ro] is a floating point value describing the absolute offset value for the right edge.
+		- [bs] is a floating point value describing the relative scale value for the bottom edge.
+		- [bo] is a floating point value describing the absolute offset value for the bottom edge.
+*/
+class UnifiedAreaRect : public Property
+{
+	public:
+		UnifiedAreaRect() : Property(
+		"UnifiedAreaRect",
+		"Property to get/set the windows unified area rectangle.  Value is a \"URect\".",
+		"{{0.000000,0.000000},{0.000000,0.000000},{0.000000,0.000000},{0.000000,0.000000}}")
+		{}
+
+		String	get(const PropertyReceiver* receiver) const;
+		void	set(PropertyReceiver* receiver, const String& value);
+};
+
+
+/*
+\brief
+	Property to access the unified position of the window.
+
+	\par Usage:
+		- Name: UnifiedPosition
+		- Format: "{{[xs],[xo]},{[ys],[yo]}}"
+
+	\par Where:
+		- [xs] is a floating point value describing the relative scale value for the position x-coordinate.
+		- [xo] is a floating point value describing the absolute offset value for the position x-coordinate.
+		- [ys] is a floating point value describing the relative scale value for the position y-coordinate.
+		- [yo] is a floating point value describing the absolute offset value for the position y-coordinate.
+*/
+class UnifiedPosition : public Property
+{
+	public:
+		UnifiedPosition() : Property(
+		"UnifiedPosition",
+		"Property to get/set the windows unified position.  Value is a \"UVector2\".",
+		"{{0.000000,0.000000},{0.000000,0.000000}}", false)
+		{}
+
+		String	get(const PropertyReceiver* receiver) const;
+		void	set(PropertyReceiver* receiver, const String& value);
+};
+
+
+/*
+\brief
+	Property to access the unified position x-coordinate of the window.
+
+	\par Usage:
+		- Name: UnifiedXPosition
+		- Format: "{[s],[o]}"
+
+	\par Where:
+		- [s] is a floating point value describing the relative scale value for the position x-coordinate.
+		- [o] is a floating point value describing the absolute offset value for the position x-coordinate.
+*/
+class UnifiedXPosition : public Property
+{
+	public:
+		UnifiedXPosition() : Property(
+		"UnifiedXPosition",
+		"Property to get/set the windows unified position x-coordinate.  Value is a \"UDim\".",
+		"{0.000000,0.000000}", false)
+		{}
+
+		String	get(const PropertyReceiver* receiver) const;
+		void	set(PropertyReceiver* receiver, const String& value);
+};
+
+
+/*
+\brief
+	Property to access the unified position y-coordinate of the window.
+
+	\par Usage:
+		- Name: UnifiedYPosition
+		- Format: "{[s],[o]}"
+
+	\par Where:
+		- [s] is a floating point value describing the relative scale value for the position y-coordinate.
+		- [o] is a floating point value describing the absolute offset value for the position y-coordinate.
+*/
+class UnifiedYPosition : public Property
+{
+	public:
+		UnifiedYPosition() : Property(
+		"UnifiedYPosition",
+		"Property to get/set the windows unified position y-coordinate.  Value is a \"UDim\".",
+		"{0.000000,0.000000}", false)
+		{}
+
+		String	get(const PropertyReceiver* receiver) const;
+		void	set(PropertyReceiver* receiver, const String& value);
+};
+
+
+/*
+\brief
+	Property to access the unified position of the window.
+
+	\par Usage:
+		- Name: UnifiedSize
+		- Format: "{{[ws],[wo]},{[hs],[ho]}}"
+
+	\par Where:
+		- [ws] is a floating point value describing the relative scale value for the width.
+		- [wo] is a floating point value describing the absolute offset value for the width.
+		- [hs] is a floating point value describing the relative scale value for the height.
+		- [ho] is a floating point value describing the absolute offset value for the height.
+*/
+class UnifiedSize : public Property
+{
+	public:
+		UnifiedSize() : Property(
+		"UnifiedSize",
+		"Property to get/set the windows unified size.  Value is a \"UVector2\".",
+		"{{0.000000,0.000000},{0.000000,0.000000}}", false)
+		{}
+
+		String	get(const PropertyReceiver* receiver) const;
+		void	set(PropertyReceiver* receiver, const String& value);
+};
+
+
+/*
+\brief
+	Property to access the unified width of the window.
+
+	\par Usage:
+		- Name: UnifiedWidth
+		- Format: "{[s],[o]}"
+
+	\par Where:
+		- [s] is a floating point value describing the relative scale value for the width.
+		- [o] is a floating point value describing the absolute offset value for the width.
+*/
+class UnifiedWidth : public Property
+{
+	public:
+		UnifiedWidth() : Property(
+		"UnifiedWidth",
+		"Property to get/set the windows unified width.  Value is a \"UDim\".",
+		"{0.000000,0.000000}", false)
+		{}
+
+		String	get(const PropertyReceiver* receiver) const;
+		void	set(PropertyReceiver* receiver, const String& value);
+};
+
+
+/*
+\brief
+	Property to access the unified height of the window.
+
+	\par Usage:
+		- Name: UnifiedHeight
+		- Format: "{[s],[o]}"
+
+	\par Where:
+		- [s] is a floating point value describing the relative scale value for the height.
+		- [o] is a floating point value describing the absolute offset value for the height.
+*/
+class UnifiedHeight : public Property
+{
+	public:
+		UnifiedHeight() : Property(
+		"UnifiedHeight",
+		"Property to get/set the windows unified height.  Value is a \"UDim\".",
+		"{0.000000,0.000000}", false)
+		{}
+
+		String	get(const PropertyReceiver* receiver) const;
+		void	set(PropertyReceiver* receiver, const String& value);
+};
+
+
+/*
+\brief
+	Property to access the unified minimum size of the window.
+
+	\par Usage:
+		- Name: UnifiedMinSize
+		- Format: "{{[ws],[wo]},{[hs],[ho]}}"
+
+	\par Where:
+		- [ws] is a floating point value describing the relative scale value for the minimum width.
+		- [wo] is a floating point value describing the absolute offset value for the minimum width.
+		- [hs] is a floating point value describing the relative scale value for the minimum height.
+		- [ho] is a floating point value describing the absolute offset value for the minimum height.
+*/
+class UnifiedMinSize : public Property
+{
+	public:
+		UnifiedMinSize() : Property(
+		"UnifiedMinSize",
+		"Property to get/set the windows unified minimum size.  Value is a \"UVector2\".",
+		"{{0.000000,0.000000},{0.000000,0.000000}}")
+		{}
+
+		String	get(const PropertyReceiver* receiver) const;
+		void	set(PropertyReceiver* receiver, const String& value);
+};
+
+
+/*
+\brief
+	Property to access the unified maximum size of the window.
+
+	\par Usage:
+		- Name: UnifiedMaxSize
+		- Format: "{{[ws],[wo]},{[hs],[ho]}}"
+
+	\par Where:
+		- [ws] is a floating point value describing the relative scale value for the maximum width.
+		- [wo] is a floating point value describing the absolute offset value for the maximum width.
+		- [hs] is a floating point value describing the relative scale value for the maximum height.
+		- [ho] is a floating point value describing the absolute offset value for the maximum height.
+*/
+class UnifiedMaxSize : public Property
+{
+	public:
+		UnifiedMaxSize() : Property(
+		"UnifiedMaxSize",
+		"Property to get/set the windows unified maximum size.  Value is a \"UVector2\".",
+		"{{0.000000,0.000000},{0.000000,0.000000}}")
+		{}
+
+		String	get(const PropertyReceiver* receiver) const;
+		void	set(PropertyReceiver* receiver, const String& value);
+};
+
 
 } // End of  WindowProperties namespace section
 
