@@ -45,7 +45,7 @@ namespace CEGUI
 \brief
 	Base class for the Combobox widget
 */
-class CEGUIBASE_API Combobox : public Window
+class CEGUIEXPORT Combobox : public Window
 {
 public:
 	static const String EventNamespace;				//!< Namespace for global events
@@ -175,7 +175,7 @@ public:
 	\return
 		Index of the insert carat relative to the start of the text.
 	*/
-	ulong	getCaratIndex(void) const;
+	size_t	getCaratIndex(void) const;
 
 
 	/*!
@@ -186,7 +186,7 @@ public:
 		Index of the selection start point relative to the start of the text.  If no selection is defined this function returns
 		the position of the carat.
 	*/
-	ulong	getSelectionStartIndex(void) const;
+	size_t	getSelectionStartIndex(void) const;
 
 
 	/*!
@@ -197,7 +197,7 @@ public:
 		Index of the selection end point relative to the start of the text.  If no selection is defined this function returns
 		the position of the carat.
 	*/
-	ulong	getSelectionEndIndex(void) const;
+	size_t	getSelectionEndIndex(void) const;
 
 	
 	/*!
@@ -207,7 +207,7 @@ public:
 	\return
 		Number of code points (or characters) contained within the currently defined selection.
 	*/
-	ulong	getSelectionLength(void) const;
+	size_t	getSelectionLength(void) const;
 
 
 	/*!
@@ -221,7 +221,7 @@ public:
 		Depending on the validation string set, the actual length of text that can be entered may be less than the value
 		returned here (it will never be more).
 	*/
-	ulong	getMaxTextLength(void) const;
+	size_t	getMaxTextLength(void) const;
 
 
 	/*!
@@ -278,7 +278,7 @@ public:
 	\return
 		the number of items currently attached to this list box.
 	*/
-	uint	getItemCount(void) const;
+	size_t	getItemCount(void) const;
 
 	
 	/*!
@@ -304,7 +304,7 @@ public:
 
 	\exception	InvalidRequestException	thrown if \a index is out of range.
 	*/
-	ListboxItem*	getListboxItemFromIndex(uint index) const;
+	ListboxItem*	getListboxItemFromIndex(size_t index) const;
 
 
 	/*!
@@ -319,7 +319,7 @@ public:
 
 	\exception	InvalidRequestException	thrown if \a item is not attached to this list box.
 	*/
-	uint	getItemIndex(const ListboxItem* item) const;
+	size_t	getItemIndex(const ListboxItem* item) const;
 
 
 	/*!
@@ -344,7 +344,7 @@ public:
 
 	\exception	InvalidRequestException	thrown if \a index is out of range.
 	*/
-	bool	isItemSelected(uint index) const;
+	bool	isItemSelected(size_t index) const;
 
 
 	/*!
@@ -494,7 +494,7 @@ public:
 	\return
 		Nothing.
 	*/
-	void	setCaratIndex(ulong carat_pos);
+	void	setCaratIndex(size_t carat_pos);
 
 
 	/*!
@@ -512,7 +512,7 @@ public:
 	\return
 		Nothing.
 	*/
-	void	setSelection(ulong start_pos, ulong end_pos);
+	void	setSelection(size_t start_pos, size_t end_pos);
 	
 
 	/*!
@@ -529,7 +529,7 @@ public:
 	\return
 		Nothing.
 	*/
-	void	setMaxTextLength(ulong max_len);
+	void	setMaxTextLength(size_t max_len);
 
 
 	/*!
@@ -746,7 +746,7 @@ public:
 	
 	\exception	InvalidRequestException	thrown if \a item_index is out of range for the list box
 	*/
-	void	setItemSelectState(uint item_index, bool state);
+	void	setItemSelectState(size_t item_index, bool state);
 
 
 	/*!
@@ -792,16 +792,6 @@ protected:
 	void	addComboboxEvents(void);
 
 
-	/*!
-	\brief
-		Setup size and position for the component widgets attached to this Combobox.
-
-	\return
-		Nothing.
-	*/
-	virtual void	layoutComponentWidgets()	= 0;
-
-
 	/*!
 	\brief
 		Create, initialise, and return a pointer to an Editbox widget to be used as part
@@ -810,7 +800,7 @@ protected:
 	\return
 		Pointer to an Editbox derived class.
 	*/
-	virtual	Editbox*	createEditbox(void) const		= 0;
+	virtual	Editbox*	createEditbox(const String& name) const		= 0;
 
 
 	/*!
@@ -821,7 +811,7 @@ protected:
 	\return
 		Pointer to a PushButton derived class.
 	*/
-	virtual	PushButton*	createPushButton(void) const	= 0;
+	virtual	PushButton*	createPushButton(const String& name) const	= 0;
 
 
 	/*!
@@ -832,7 +822,7 @@ protected:
 	\return
 		Pointer to a ComboDropList derived class.
 	*/
-	virtual	ComboDropList*	createDropList(void) const	= 0;
+	virtual	ComboDropList*	createDropList(const String& name) const	= 0;
 
 
 	/*!
@@ -863,6 +853,23 @@ protected:
 	bool	editbox_MouseDownHandler(const EventArgs& e);
 
 
+	/*!
+	\brief
+		Return whether this window was inherited from the given class name at some point in the inheritance heirarchy.
+
+	\param class_name
+		The class name that is to be checked.
+
+	\return
+		true if this window was inherited from \a class_name. false if not.
+	*/
+	virtual bool	testClassName_impl(const String& class_name) const
+	{
+		if (class_name==(const utf8*)"Combobox")	return true;
+		return Window::testClassName_impl(class_name);
+	}
+
+
 	/*************************************************************************
 		Handlers to relay child widget events so they appear to come from us
 	*************************************************************************/
@@ -1012,7 +1019,6 @@ protected:
 		Overridden Event handlers
 	*************************************************************************/
 	virtual	void	onFontChanged(WindowEventArgs& e);
-    virtual	void	onSized(WindowEventArgs& e);
 	virtual void	onTextChanged(WindowEventArgs& e);
 	virtual void	onActivated(ActivationEventArgs& e);
 
