@@ -46,7 +46,7 @@ namespace CEGUI
 \brief
 	Base class for standard Listbox widget.
 */
-class CEGUIBASE_API Listbox : public Window
+class CEGUIEXPORT Listbox : public Window
 {
 public:
 	static const String EventNamespace;				//!< Namespace for global events
@@ -74,7 +74,7 @@ public:
 	\return
 		the number of items currently attached to this list box.
 	*/
-	uint	getItemCount(void) const		{return (uint)d_listItems.size();}
+	size_t	getItemCount(void) const		{return d_listItems.size();}
 
 	
 	/*!
@@ -84,7 +84,7 @@ public:
 	\return
 		Total number of attached items that are in the selected state.
 	*/
-	uint	getSelectedCount(void) const;
+	size_t	getSelectedCount(void) const;
 
 
 	/*!
@@ -127,7 +127,7 @@ public:
 
 	\exception	InvalidRequestException	thrown if \a index is out of range.
 	*/
-	ListboxItem*	getListboxItemFromIndex(uint index) const;
+	ListboxItem*	getListboxItemFromIndex(size_t index) const;
 
 
 	/*!
@@ -142,7 +142,7 @@ public:
 
 	\exception	InvalidRequestException	thrown if \a item is not attached to this list box.
 	*/
-	uint	getItemIndex(const ListboxItem* item) const;
+	size_t	getItemIndex(const ListboxItem* item) const;
 
 
 	/*!
@@ -163,6 +163,7 @@ public:
 	*/
 	bool	isMultiselectEnabled(void) const	{return d_multiselect;}
 
+	bool	isItemTooltipsEnabled(void) const	{return d_itemTooltips;}
 
 	/*!
 	\brief
@@ -176,7 +177,7 @@ public:
 
 	\exception	InvalidRequestException	thrown if \a index is out of range.
 	*/
-	bool	isItemSelected(uint index) const;
+	bool	isItemSelected(size_t index) const;
 
 
 	/*!
@@ -286,6 +287,8 @@ public:
 
 	\return
 		Nothing.
+	
+	\exception InvalidRequestException	thrown if no ListboxItem \a position is attached to this list box.
 	*/
 	void	insertItem(ListboxItem* item, const ListboxItem* position);
 
@@ -368,7 +371,7 @@ public:
 	*/
 	void	setShowHorzScrollbar(bool setting);
 
-
+	void	setItemTooltipsEnabled(bool setting);
 	/*!
 	\brief
 		Set the select state of an attached ListboxItem.
@@ -410,7 +413,7 @@ public:
 	
 	\exception	InvalidRequestException	thrown if \a item_index is out of range for the list box
 	*/
-	void	setItemSelectState(uint item_index, bool state);
+	void	setItemSelectState(size_t item_index, bool state);
 
 
 	/*!
@@ -439,7 +442,7 @@ public:
 	\return
 		Nothing.
 	*/
-	void	ensureItemIsVisible(uint item_index);
+	void	ensureItemIsVisible(size_t item_index);
 
 
 	/*!
@@ -494,46 +497,39 @@ protected:
 	\brief
 		create and return a pointer to a Scrollbar widget for use as vertical scroll bar
 
+	\param name
+	   String holding the name to be given to the created widget component.
+
 	\return
 		Pointer to a Scrollbar to be used for scrolling the list vertically.
 	*/
-	virtual Scrollbar*	createVertScrollbar(void) const		= 0;
+	virtual Scrollbar*	createVertScrollbar(const String& name) const		= 0;
  
 
 	/*!
 	\brief
 		create and return a pointer to a Scrollbar widget for use as horizontal scroll bar
 
+	\param name
+	   String holding the name to be given to the created widget component.
+
 	\return
 		Pointer to a Scrollbar to be used for scrolling the list horizontally.
 	*/
-	virtual Scrollbar*	createHorzScrollbar(void) const		= 0;
+	virtual Scrollbar*	createHorzScrollbar(const String& name) const		= 0;
 
 
 	/*!
 	\brief
-		Setup size and position for the component widgets attached to this Listbox
-
-	\return
-		Nothing.
-	*/
-	virtual void	layoutComponentWidgets()	= 0;
-
-
-	/*!
-	\brief
-		Perform rendering of the widget control frame and other 'static' areas.  This
+		Perform caching of the widget control frame and other 'static' areas.  This
 		method should not render the actual items.  Note that the items are typically
 		rendered to layer 3, other layers can be used for rendering imagery behind and
 		infront of the items.
 
-	\param z
-		Z co-ordinate for layer 0.
-
 	\return
 		Nothing.
 	*/
-	virtual	void	renderListboxBaseImagery(float z)		= 0;
+	virtual	void cacheListboxBaseImagery() = 0;
 
 
 	/*************************************************************************
@@ -546,19 +542,6 @@ protected:
 	void	addListboxEvents(void);
 
 
-	/*!
-	\brief
-		Perform the actual rendering for this Window.
-
-	\param z
-		float value specifying the base Z co-ordinate that should be used when rendering
-
-	\return
-		Nothing
-	*/
-	virtual	void	drawSelf(float z);
-
-
 	/*!
 	\brief
 		display required integrated scroll bars according to current state of the list box and update their values.
@@ -570,7 +553,7 @@ protected:
 		select all strings between positions \a start and \a end.  (inclusive)
 		including \a end.
 	*/
-	void	selectRange(uint start, uint end);
+	void	selectRange(size_t start, size_t end);
 
 
 	/*!
@@ -622,6 +605,32 @@ protected:
 	bool	resetList_impl(void);
 
 
+	/*!
+	\brief
+		Return whether this window was inherited from the given class name at some point in the inheritance heirarchy.
+
+	\param class_name
+		The class name that is to be checked.
+
+	\return
+		true if this window was inherited from \a class_name. false if not.
+	*/
+	virtual bool	testClassName_impl(const String& class_name) const
+	{
+		if (class_name==(const utf8*)"Listbox")	return true;
+		return Window::testClassName_impl(class_name);
+	}
+
+	/*!
+	\brief
+	   Internal handler that is triggered when the user interacts with the scrollbars.
+    */
+    bool handle_scrollChange(const EventArgs& args);
+
+    // overridden from Window base class.
+    void populateRenderCache();
+
+
 	/*************************************************************************
 		New event handlers
 	*************************************************************************/
@@ -673,6 +682,7 @@ protected:
 	virtual void	onSized(WindowEventArgs& e);
 	virtual void	onMouseButtonDown(MouseEventArgs& e);
 	virtual	void	onMouseWheel(MouseEventArgs& e);
+	virtual void	onMouseMove(MouseEventArgs& e);
 
 
 	/*************************************************************************
@@ -683,6 +693,7 @@ protected:
 	bool	d_multiselect;			//!< true if multi-select is enabled
 	bool	d_forceVertScroll;		//!< true if vertical scrollbar should always be displayed
 	bool	d_forceHorzScroll;		//!< true if horizontal scrollbar should always be displayed
+	bool	d_itemTooltips;			//!< true if each item should have an individual tooltip
 	Scrollbar*	d_vertScrollbar;	//!< vertical scroll-bar widget
 	Scrollbar*	d_horzScrollbar;	//!< horizontal scroll-bar widget
 	LBItemList	d_listItems;		//!< list of items in the list box.
@@ -697,6 +708,7 @@ private:
 	static ListboxProperties::MultiSelect			d_multiSelectProperty;
 	static ListboxProperties::ForceVertScrollbar	d_forceVertProperty;
 	static ListboxProperties::ForceHorzScrollbar	d_forceHorzProperty;
+	static ListboxProperties::ItemTooltips			d_itemTooltipsProperty;
 
 	/*************************************************************************
 		Private methods
@@ -710,7 +722,7 @@ private:
 	Helper function used in sorting to compare two list box item text strings
 	via the ListboxItem pointers and return if \a a is less than \a b.
 */
-static bool lbi_less(const ListboxItem* a, const ListboxItem* b);
+bool lbi_less(const ListboxItem* a, const ListboxItem* b);
 
 
 /*!
@@ -718,7 +730,7 @@ static bool lbi_less(const ListboxItem* a, const ListboxItem* b);
 	Helper function used in sorting to compare two list box item text strings
 	via the ListboxItem pointers and return if \a a is greater than \a b.
 */
-static bool lbi_greater(const ListboxItem* a, const ListboxItem* b);
+bool lbi_greater(const ListboxItem* a, const ListboxItem* b);
 
 } // End of  CEGUI namespace section
 
