@@ -46,7 +46,7 @@ namespace CEGUI
 \brief
 	Simple grid index structure.
 */
-struct CEGUIBASE_API MCLGridRef
+struct CEGUIEXPORT MCLGridRef
 {
 	MCLGridRef(uint r, uint c) : row(r), column(c) {}
 
@@ -68,7 +68,7 @@ struct CEGUIBASE_API MCLGridRef
 \brief
 	Base class for the multi column list widget.
 */
-class CEGUIBASE_API MultiColumnList : public Window
+class CEGUIEXPORT MultiColumnList : public Window
 {
 public:
 	static const String EventNamespace;				//!< Namespace for global events
@@ -575,6 +575,36 @@ public:
 	uint	getColumnID(uint col_idx) const;
 
 
+	/*!
+	\brief
+		Return the ID code assigned to the requested row.
+
+	\param row_idx
+		Zero based index of the row who's ID code is to be returned.
+
+	\return
+		Current ID code assigned to the row at the requested index.
+
+	\exception InvalidRequestException	thrown if \a row_idx is out of range
+	*/
+	uint	getRowID(uint row_idx) const;
+
+
+	/*!
+	\brief
+		Return the zero based row index of the row with the specified ID.
+
+	\param row_id
+		ID code of the row who's index is to be returned.
+
+	\return
+		Zero based row index of the first row who's ID matches \a row_id.
+
+	\exception	InvalidRequestException		thrown if no row has the requested ID.
+	*/
+	uint	getRowWithID(uint row_id) const;
+
+
 	/*************************************************************************
 		Manipulator Methods
 	*************************************************************************/
@@ -712,6 +742,9 @@ public:
 	\brief
 		Add an empty row to the list box.
 
+	\param row_id
+		ID code to be assigned to the new row.
+
 	\note
 		If the list is being sorted, the new row will appear at an appropriate position according to the sorting being
 		applied.  If no sorting is being done, the new row will appear at the bottom of the list.
@@ -719,7 +752,7 @@ public:
 	\return
 		Initial zero based index of the new row.
 	*/
-	uint	addRow(void);
+	uint	addRow(uint row_id = 0);
 
 
 	/*!
@@ -736,12 +769,15 @@ public:
 	\param col_id
 		ID code of the column whos initial item is to be set to \a item.
 
+	\param row_id
+		ID code to be assigned to the new row.
+
 	\return
 		Initial zero based index of the new row.
 
 	\exception InvalidRequestException	thrown if no column with the specified ID is attached to the list box.
 	*/
-	uint	addRow(ListboxItem* item, uint col_id);
+	uint	addRow(ListboxItem* item, uint col_id, uint row_id = 0);
 
 
 	/*!
@@ -756,10 +792,13 @@ public:
 		Zero based index where the row should be inserted.  If this is greater than the current number of rows, the row is
 		appended to the list.
 
+	\param row_id
+		ID code to be assigned to the new row.
+
 	\return
 		Zero based index where the row was actually inserted.
 	*/
-	uint	insertRow(uint row_idx);
+	uint	insertRow(uint row_idx, uint row_id = 0);
 
 
 	/*!
@@ -780,12 +819,15 @@ public:
 		Zero based index where the row should be inserted.  If this is greater than the current number of rows, the row is
 		appended to the list.
 
+	\param row_id
+		ID code to be assigned to the new row.
+
 	\return
 		Zero based index where the row was actually inserted.
 
 	\exception InvalidRequestException	thrown if no column with the specified ID is attached to the list box.
 	*/
-	uint	insertRow(ListboxItem* item, uint col_id, uint row_idx);
+	uint	insertRow(ListboxItem* item, uint col_id, uint row_idx, uint row_id = 0);
 
 
 	/*!
@@ -1113,6 +1155,24 @@ public:
 	void	autoSizeColumnHeader(uint col_idx);
 
 
+	/*!
+	\brief
+		Set the ID code assigned to a given row.
+
+	\param row_idx
+		Zero based index of the row who's ID code is to be set.
+
+	\param row_id
+		ID code to be assigned to the row at the requested index.
+
+	\return
+		Nothing.
+
+	\exception InvalidRequestException	thrown if \a row_idx is out of range
+	*/
+	void	setRowID(uint row_idx, uint row_id);
+
+
 	/*************************************************************************
 		Construction and Destruction
 	*************************************************************************/
@@ -1150,40 +1210,39 @@ protected:
 	\brief
 		create and return a pointer to a ListHeaer widget for use as the column headers.
 
+	\param name
+	   String holding the name to be assigned to the created component.
+
 	\return
 		Pointer to a ListHeader based object.
 	*/
-	virtual ListHeader*	createListHeader(void) const		= 0;
+	virtual ListHeader*	createListHeader(const String& name) const		= 0;
  
 
 	/*!
 	\brief
 		create and return a pointer to a Scrollbar widget for use as vertical scroll bar
 
+	\param name
+	   String holding the name to be assigned to the created component.
+
 	\return
 		Pointer to a Scrollbar to be used for scrolling the list vertically.
 	*/
-	virtual Scrollbar*	createVertScrollbar(void) const		= 0;
+	virtual Scrollbar*	createVertScrollbar(const String& name) const		= 0;
  
 
 	/*!
 	\brief
 		create and return a pointer to a Scrollbar widget for use as horizontal scroll bar
 
+	\param name
+	   String holding the name to be assigned to the created component.
+
 	\return
 		Pointer to a Scrollbar to be used for scrolling the list horizontally.
 	*/
-	virtual Scrollbar*	createHorzScrollbar(void) const		= 0;
-
-
-	/*!
-	\brief
-		Setup size and position for the component widgets attached to this Listbox
-
-	\return
-		Nothing.
-	*/
-	virtual void	layoutComponentWidgets()	= 0;
+	virtual Scrollbar*	createHorzScrollbar(const String& name) const		= 0;
 
 
 	/*!
@@ -1193,13 +1252,10 @@ protected:
 		rendered to layer 3, other layers can be used for rendering imagery behind and
 		infront of the items.
 
-	\param z
-		Z co-ordinate for layer 0.
-
 	\return
 		Nothing.
 	*/
-	virtual	void	renderListboxBaseImagery(float z)		= 0;
+	virtual	void	cacheListboxBaseImagery()		= 0;
 
 
 	/*************************************************************************
@@ -1211,18 +1267,8 @@ protected:
 	*/
 	void	addMultiColumnListboxEvents(void);
 
-
-	/*!
-	\brief
-		Perform the actual rendering for this Window.
-
-	\param z
-		float value specifying the base Z co-ordinate that should be used when rendering
-
-	\return
-		Nothing
-	*/
-	virtual	void	drawSelf(float z);
+    // overridden from base class.
+	virtual	void populateRenderCache();
 
 
 	/*!
@@ -1328,6 +1374,25 @@ protected:
 	bool	resetList_impl(void);
 
 
+	/*!
+	\brief
+		Return whether this window was inherited from the given class name at some point in the inheritance heirarchy.
+
+	\param class_name
+		The class name that is to be checked.
+
+	\return
+		true if this window was inherited from \a class_name. false if not.
+	*/
+	virtual bool	testClassName_impl(const String& class_name) const
+	{
+		if (class_name==(const utf8*)"MultiColumnList")	return true;
+		return Window::testClassName_impl(class_name);
+	}
+
+    // overrides function in base class.
+    int writePropertiesXML(OutStream& out_stream) const;
+
 	/*************************************************************************
 		New event handlers for multi column list
 	*************************************************************************/
@@ -1423,19 +1488,22 @@ protected:
 	bool	handleHeaderSegMove(const EventArgs& e);
 	bool	handleColumnSizeChange(const EventArgs& e);
 	bool	handleHorzScrollbar(const EventArgs& e);
+	bool	handleVertScrollbar(const EventArgs& e);
 	bool	handleSortColumnChange(const EventArgs& e);
 	bool	handleSortDirectionChange(const EventArgs& e);
 	bool	handleHeaderSegDblClick(const EventArgs& e);
 
-
-	/*************************************************************************
-		Struct used to wrap a 'row' and ease sorting
-	*************************************************************************/
+    /*!
+    \brief
+        Struct used internally to represent a row in the list and also to ease
+        sorting of the rows.
+    */
 	struct ListRow
 	{
 		typedef	std::vector<ListboxItem*>	RowItems;
 		RowItems	d_items;
 		uint		d_sortColumn;
+		uint		d_rowID;
 
 		// operators
 		ListboxItem* const& operator[](uint idx) const	{return d_items[idx];}
@@ -1492,6 +1560,8 @@ private:
 	static MultiColumnListProperties::SortColumnID					d_sortColumnIDProperty;
 	static MultiColumnListProperties::SortDirection					d_sortDirectionProperty;
 	static MultiColumnListProperties::SortSettingEnabled			d_sortSettingProperty;
+	static MultiColumnListProperties::ColumnHeader					d_columnHeaderProperty;
+	static MultiColumnListProperties::RowCount						d_rowCountProperty;
 
 
 	/*************************************************************************
