@@ -47,7 +47,7 @@ namespace CEGUI
 \brief
 	Base class for the multi-line edit box widget.
 */
-class CEGUIBASE_API MultiLineEditbox : public Window
+class CEGUIEXPORT MultiLineEditbox : public Window
 {
 public:
 	static const String EventNamespace;				//!< Namespace for global events
@@ -67,10 +67,10 @@ public:
 	static const String EventHorzScrollbarModeChanged;	//!< Event triggered when the horizontal scroll bar 'force' setting changes.
 
 	// default colours
-	static const ulong	DefaultNormalTextColour;			//!< Colour applied to normal unselected text.
-	static const ulong	DefaultSelectedTextColour;			//!< Colour applied to selected text.
-	static const ulong	DefaultNormalSelectionColour;		//!< Colour applied to normal selection brush.
-	static const ulong	DefaultInactiveSelectionColour;		//!< Colour applied to selection brush when widget is inactive.
+	static const argb_t	DefaultNormalTextColour;			//!< Colour applied to normal unselected text.
+	static const argb_t	DefaultSelectedTextColour;			//!< Colour applied to selected text.
+	static const argb_t	DefaultNormalSelectionColour;		//!< Colour applied to normal selection brush.
+	static const argb_t	DefaultInactiveSelectionColour;		//!< Colour applied to selection brush when widget is inactive.
 
 
 	/*************************************************************************
@@ -105,7 +105,7 @@ public:
 	\return
 		Index of the insert carat relative to the start of the text.
 	*/
-	ulong	getCaratIndex(void) const		{return d_caratPos;}
+	size_t	getCaratIndex(void) const		{return d_caratPos;}
 
 
 	/*!
@@ -116,7 +116,7 @@ public:
 		Index of the selection start point relative to the start of the text.  If no selection is defined this function returns
 		the position of the carat.
 	*/
-	ulong	getSelectionStartIndex(void) const;
+	size_t	getSelectionStartIndex(void) const;
 
 
 	/*!
@@ -127,7 +127,7 @@ public:
 		Index of the selection end point relative to the start of the text.  If no selection is defined this function returns
 		the position of the carat.
 	*/
-	ulong	getSelectionEndIndex(void) const;
+	size_t	getSelectionEndIndex(void) const;
 
 	
 	/*!
@@ -137,7 +137,7 @@ public:
 	\return
 		Number of code points (or characters) contained within the currently defined selection.
 	*/
-	ulong	getSelectionLength(void) const;
+	size_t	getSelectionLength(void) const;
 
 
 	/*!
@@ -147,7 +147,7 @@ public:
 	\return
 		The maximum number of code points (characters) that can be entered into this edit box.
 	*/
-	ulong	getMaxTextLength(void) const		{return d_maxTextLen;}
+	size_t	getMaxTextLength(void) const		{return d_maxTextLen;}
 
 
 	/*!
@@ -246,7 +246,7 @@ public:
 	\return
 		Nothing.
 	*/
-	void	setCaratIndex(ulong carat_pos);
+	void	setCaratIndex(size_t carat_pos);
 
 
 	/*!
@@ -264,7 +264,7 @@ public:
 	\return
 		Nothing.
 	*/
-	void	setSelection(ulong start_pos, ulong end_pos);
+	void	setSelection(size_t start_pos, size_t end_pos);
 	
 
 	/*!
@@ -277,7 +277,7 @@ public:
 	\return
 		Nothing.
 	*/
-	void	setMaxTextLength(ulong max_len);
+	void	setMaxTextLength(size_t max_len);
 
 
 	/*!
@@ -389,30 +389,26 @@ protected:
 	\brief
 		create and return a pointer to a Scrollbar widget for use as vertical scroll bar
 
+	\param name
+	   String holding the name to be assigned to the component.
+
 	\return
 		Pointer to a Scrollbar to be used for scrolling vertically.
 	*/
-	virtual Scrollbar*	createVertScrollbar(void) const		= 0;
+	virtual Scrollbar*	createVertScrollbar(const String& name) const		= 0;
  
 
 	/*!
 	\brief
 		create and return a pointer to a Scrollbar widget for use as horizontal scroll bar
 
+	\param name
+	   String holding the name to be assigned to the component.
+
 	\return
 		Pointer to a Scrollbar to be used for scrolling horizontally.
 	*/
-	virtual Scrollbar*	createHorzScrollbar(void) const		= 0;
-
-
-	/*!
-	\brief
-		Setup size and position for the component widgets attached to this Editbox
-
-	\return
-		Nothing.
-	*/
-	virtual void	layoutComponentWidgets()	= 0;
+	virtual Scrollbar*	createHorzScrollbar(const String& name) const		= 0;
 
 
 	/*!
@@ -422,13 +418,10 @@ protected:
 		to layer 4 and the selection brush to layer 3, other layers can be used for
 		rendering imagery behind and infront of the text & selection..
 
-	\param z
-		Z co-ordinate for layer 0.
-
 	\return
 		Nothing.
 	*/
-	virtual	void	renderEditboxBaseImagery(float z)		= 0;
+	virtual	void	cacheEditboxBaseImagery()		= 0;
 
 
 	/*!
@@ -438,7 +431,7 @@ protected:
 	\return
 		Nothing
 	*/
-	virtual void	renderCarat(float baseX, float baseY, const Rect& clipper)	= 0;
+	virtual void	cacheCaratImagery(const Rect& textArea)	= 0;
 
 
 	/*************************************************************************
@@ -455,7 +448,7 @@ protected:
 	\brief
 		Render text lines.
 	*/
-	void	renderTextLines(const Rect& dest_area, const Rect& clipper) const;
+	void	cacheTextLines(const Rect& dest_area);
 
 
 	/*!
@@ -475,20 +468,10 @@ protected:
 	\return
 		The code point length of the token.
 	*/
-	uint	getNextTokenLength(const String& text, uint start_idx) const;
+	size_t	getNextTokenLength(const String& text, size_t start_idx) const;
 
 
-	/*!
-	\brief
-		Perform the actual rendering for this Window.
-
-	\param z
-		float value specifying the base Z co-ordinate that should be used when rendering
-
-	\return
-		Nothing
-	*/
-	virtual	void	drawSelf(float z);
+	virtual	void populateRenderCache();
 
 
 	/*!
@@ -508,7 +491,7 @@ protected:
 	\return
 		Code point index into the text that is rendered closest to screen position \a pt.
 	*/
-	uint	getTextIndexFromPosition(const Point& pt) const;
+	size_t	getTextIndexFromPosition(const Point& pt) const;
 
 
 	/*!
@@ -516,7 +499,7 @@ protected:
 		Return the line number a given index falls on with the current formatting.  Will return last line
 		if index is out of range.
 	*/
-	uint	getLineNumberFromIndex(uint index) const;
+	size_t	getLineNumberFromIndex(size_t index) const;
 
 
 	/*!
@@ -627,6 +610,34 @@ protected:
 	void	handleNewLine(uint sysKeys);
 
 
+	/*!
+	\brief
+		Return whether this window was inherited from the given class name at some point in the inheritance heirarchy.
+
+	\param class_name
+		The class name that is to be checked.
+
+	\return
+		true if this window was inherited from \a class_name. false if not.
+	*/
+	virtual bool	testClassName_impl(const String& class_name) const
+	{
+		if ((class_name==(const utf8*)"MultiLineEditBox") ||
+			(class_name==(const utf8*)"MultiLineEditbox"))
+		{
+			return true;
+		}
+
+		return Window::testClassName_impl(class_name);
+	}
+
+	/*!
+	\brief
+	   Internal handler that is triggered when the user interacts with the scrollbars.
+    */
+    bool handle_scrollChange(const EventArgs& args);
+
+
 	/*************************************************************************
 		New event handlers
 	*************************************************************************/
@@ -712,8 +723,8 @@ protected:
 	*/
 	struct LineInfo
 	{
-		ulong	d_startIdx;		//!< Starting index for this line.
-		ulong	d_length;		//!< Code point length of this line.
+		size_t	d_startIdx;		//!< Starting index for this line.
+		size_t	d_length;		//!< Code point length of this line.
 		float	d_extent;		//!< Rendered extent of this line.
 	};
 
@@ -722,12 +733,12 @@ protected:
 		Implementation data
 	*************************************************************************/
 	bool	d_readOnly;			//!< true if the edit box is in read-only mode
-	ulong	d_maxTextLen;		//!< Maximum number of characters for this Editbox.
-	ulong	d_caratPos;			//!< Position of the carat / insert-point.
-	ulong	d_selectionStart;	//!< Start of selection area.
-	ulong	d_selectionEnd;		//!< End of selection area.
+	size_t	d_maxTextLen;		//!< Maximum number of characters for this Editbox.
+	size_t	d_caratPos;			//!< Position of the carat / insert-point.
+	size_t	d_selectionStart;	//!< Start of selection area.
+	size_t	d_selectionEnd;		//!< End of selection area.
 	bool	d_dragging;			//!< true when a selection is being dragged.
-	ulong	d_dragAnchorIdx;	//!< Selection index for drag selection anchor point.
+	size_t	d_dragAnchorIdx;	//!< Selection index for drag selection anchor point.
 
 	typedef	std::vector<LineInfo>	LineList;	//!< Type for collection of LineInfos.
 	static String d_lineBreakChars;	//!< Holds what we consider to be line break characters.
