@@ -35,10 +35,8 @@
 #include "CEGUIFont_xmlHandler.h"
 #include "CEGUIFont_implData.h"
 #include "CEGUIResourceProvider.h"
+#include "CEGUIXMLParser.h"
 
-#include "xercesc/sax2/SAX2XMLReader.hpp"
-#include "xercesc/sax2/XMLReaderFactory.hpp"
-#include "xercesc/framework/MemBufInputSource.hpp"
 #include <ft2build.h>
 #include FT_FREETYPE_H
 
@@ -51,7 +49,7 @@ namespace CEGUI
 /*************************************************************************
 	static data definitions
 *************************************************************************/
-const uint32 Font::DefaultColour					= 0xFFFFFFFF;
+const argb_t Font::DefaultColour					= 0xFFFFFFFF;
 const uint	Font::InterGlyphPadSpace			= 2;
 
 // XML related strings
@@ -61,17 +59,22 @@ const char	Font::FontSchemaName[]				= "Font.xsd";
 /*************************************************************************
 	Constructs a new Font object from a font definition file
 *************************************************************************/
-Font::Font(const String& filename, const String& resourceGroup, FontImplData* dat)
+Font::Font(const String& filename, const String& resourceGroup, FontImplData* dat) :
+    d_glyph_images(0),
+    d_freetype(false),
+    d_lineHeight(0),
+    d_lineSpacing(0),
+    d_max_bearingY(0),
+    d_maxGlyphHeight(0),
+    d_impldat(dat),
+    d_ptSize(0),
+    d_autoScale(false),
+    d_horzScaling(1.0f),
+    d_vertScaling(1.0f),
+    d_nativeHorzRes(DefaultNativeHorzRes),
+    d_nativeVertRes(DefaultNativeVertRes),
+    d_antiAliased(false)
 {
-	d_antiAliased = false;
-	d_impldat = dat;
-	d_freetype = false;
-	d_glyph_images = NULL;
-
-	// defaults for scaling options
-	d_autoScale = false;
-	setNativeResolution(Size(DefaultNativeHorzRes, DefaultNativeVertRes));
-
 	load(filename, resourceGroup);
 
 	// complete y spacing set-up for bitmap / static fonts
@@ -84,16 +87,22 @@ Font::Font(const String& filename, const String& resourceGroup, FontImplData* da
 	'glyph-set' describes the set of code points to be available via
 	this font
 *************************************************************************/
-Font::Font(const String& name, const String& fontname, const String& resourceGroup, uint size, uint flags, const String& glyph_set, FontImplData* dat)
+Font::Font(const String& name, const String& fontname, const String& resourceGroup, uint size, uint flags, const String& glyph_set, FontImplData* dat) :
+    d_glyph_images(0),
+    d_freetype(false),
+    d_lineHeight(0),
+    d_lineSpacing(0),
+    d_max_bearingY(0),
+    d_maxGlyphHeight(0),
+    d_impldat(dat),
+    d_ptSize(0),
+    d_autoScale(false),
+    d_horzScaling(1.0f),
+    d_vertScaling(1.0f),
+    d_nativeHorzRes(DefaultNativeHorzRes),
+    d_nativeVertRes(DefaultNativeVertRes),
+    d_antiAliased(false)
 {
-	d_impldat = dat;
-	d_freetype = false;
-	d_glyph_images = NULL;
-
-	// defaults for scaling options
-	d_autoScale = false;
-	setNativeResolution(Size(DefaultNativeHorzRes, DefaultNativeVertRes));
-
 	constructor_impl(name, fontname, resourceGroup, size, flags, glyph_set);
 }
 
@@ -103,16 +112,22 @@ Font::Font(const String& name, const String& fontname, const String& resourceGro
 	[first_code_point, last_code_point] describes the range of code
 	points to be available via this font
 *************************************************************************/
-Font::Font(const String& name, const String& fontname, const String& resourceGroup, uint size, uint flags, utf32 first_code_point, utf32 last_code_point, FontImplData* dat)
+Font::Font(const String& name, const String& fontname, const String& resourceGroup, uint size, uint flags, utf32 first_code_point, utf32 last_code_point, FontImplData* dat) :
+    d_glyph_images(0),
+    d_freetype(false),
+    d_lineHeight(0),
+    d_lineSpacing(0),
+    d_max_bearingY(0),
+    d_maxGlyphHeight(0),
+    d_impldat(dat),
+    d_ptSize(0),
+    d_autoScale(false),
+    d_horzScaling(1.0f),
+    d_vertScaling(1.0f),
+    d_nativeHorzRes(DefaultNativeHorzRes),
+    d_nativeVertRes(DefaultNativeVertRes),
+    d_antiAliased(false)
 {
-	d_impldat = dat;
-	d_freetype = false;
-	d_glyph_images = NULL;
-
-	// defaults for scaling options
-	d_autoScale = false;
-	setNativeResolution(Size(DefaultNativeHorzRes, DefaultNativeVertRes));
-
 	String tmp;
 
 	for (utf32 cp = first_code_point; cp <= last_code_point; ++cp)
@@ -128,16 +143,22 @@ Font::Font(const String& name, const String& fontname, const String& resourceGro
 	Constructs a new Font object (via FreeType & a true-type font file)
 	The font file will provide support for 7-bit ASCII characters only
 *************************************************************************/
-Font::Font(const String& name, const String& fontname, const String& resourceGroup, uint size, uint flags, FontImplData* dat)
+Font::Font(const String& name, const String& fontname, const String& resourceGroup, uint size, uint flags, FontImplData* dat) :
+    d_glyph_images(0),
+    d_freetype(false),
+    d_lineHeight(0),
+    d_lineSpacing(0),
+    d_max_bearingY(0),
+    d_maxGlyphHeight(0),
+    d_impldat(dat),
+    d_ptSize(0),
+    d_autoScale(false),
+    d_horzScaling(1.0f),
+    d_vertScaling(1.0f),
+    d_nativeHorzRes(DefaultNativeHorzRes),
+    d_nativeVertRes(DefaultNativeVertRes),
+    d_antiAliased(false)
 {
-	d_impldat = dat;
-	d_freetype = false;
-	d_glyph_images = NULL;
-
-	// defaults for scaling options
-	d_autoScale = false;
-	setNativeResolution(Size(DefaultNativeHorzRes, DefaultNativeVertRes));
-
 	String tmp;
 
 	for (utf32 cp = 32; cp <= 127; ++cp)
@@ -164,23 +185,30 @@ Font::~Font(void)
 *************************************************************************/
 float Font::getTextExtent(const String& text, float x_scale) const
 {
-	float cur_extent = 0;
+    float cur_extent = 0, adv_extent = 0, width;
 
-	uint char_count = text.length();
-	CodepointMap::const_iterator	pos, end = d_cp_map.end();
+    size_t char_count = text.length();
+    CodepointMap::const_iterator pos, end = d_cp_map.end();
 
-	for (uint c = 0; c < char_count; ++c)
-	{
-		pos = d_cp_map.find(text[c]);
+    for (size_t c = 0; c < char_count; ++c)
+    {
+        pos = d_cp_map.find(text[c]);
 
-		if (pos != end)
-		{
-			cur_extent += (float)pos->second.d_horz_advance * x_scale;
-		}
+        if (pos != end)
+        {
+            width = (pos->second.d_image->getWidth() + pos->second.d_image->getOffsetX()) * x_scale;
+            
+            if (adv_extent + width > cur_extent)
+            {
+                cur_extent = adv_extent + width;
+            }
+            
+            adv_extent += (float)pos->second.d_horz_advance * x_scale;
+        }
 
-	}
+    }
 
-	return cur_extent;
+    return ceguimax(adv_extent, cur_extent);
 }
 
 
@@ -188,10 +216,10 @@ float Font::getTextExtent(const String& text, float x_scale) const
 	Return the index of the closest text character in String 'text' that
 	corresponds to pixel location 'pixel' if the text were rendered.
 *************************************************************************/
-uint Font::getCharAtPixel(const String& text, uint start_char, float pixel, float x_scale) const
+size_t Font::getCharAtPixel(const String& text, size_t start_char, float pixel, float x_scale) const
 {
 	float cur_extent = 0;
-	uint char_count = text.length();
+	size_t char_count = text.length();
 
 	// handle simple cases
 	if ((pixel <= 0) || (char_count <= start_char))
@@ -201,7 +229,7 @@ uint Font::getCharAtPixel(const String& text, uint start_char, float pixel, floa
 
 	CodepointMap::const_iterator	pos, end = d_cp_map.end();
 
-	for (uint c = start_char; c < char_count; ++c)
+	for (size_t c = start_char; c < char_count; ++c)
 	{
 		pos = d_cp_map.find(text[c]);
 
@@ -225,10 +253,10 @@ uint Font::getCharAtPixel(const String& text, uint start_char, float pixel, floa
 /*************************************************************************
 	Renders text on the display.  Return number of lines output.
 *************************************************************************/
-uint Font::drawText(const String& text, const Rect& draw_area, float z, const Rect& clip_rect, TextFormatting fmt, const ColourRect& colours, float x_scale, float y_scale) const
+size_t Font::drawText(const String& text, const Rect& draw_area, float z, const Rect& clip_rect, TextFormatting fmt, const ColourRect& colours, float x_scale, float y_scale) const
 {
-	uint thisCount;
-	uint lineCount = 0;
+	size_t thisCount;
+	size_t lineCount = 0;
 
 	float	y_base = draw_area.d_top + getBaseline(y_scale);
 
@@ -239,7 +267,7 @@ uint Font::drawText(const String& text, const Rect& draw_area, float z, const Re
 		PixelAligned(draw_area.d_bottom)
 		);
 
-	uint lineStart = 0, lineEnd = 0;
+	size_t lineStart = 0, lineEnd = 0;
 	String	currLine;
 
 	while (lineEnd < text.length())
@@ -272,6 +300,13 @@ uint Font::drawText(const String& text, const Rect& draw_area, float z, const Re
 			y_base += getLineSpacing(y_scale);
 			break;
 
+		case Justified:
+			// new function in order to keep drawTextLine's signature unchanged
+			drawTextLineJustified(currLine, draw_area, Vector3(tmpDrawArea.d_left, y_base, z), clip_rect, colours, x_scale, y_scale);
+			thisCount = 1;
+			y_base += getLineSpacing(y_scale);
+			break;
+
 		case WordWrapLeftAligned:
 			thisCount = drawWrappedText(currLine, tmpDrawArea, z, clip_rect, LeftAligned, colours, x_scale, y_scale);
 			tmpDrawArea.d_top += thisCount * getLineSpacing(y_scale);
@@ -287,6 +322,12 @@ uint Font::drawText(const String& text, const Rect& draw_area, float z, const Re
 			tmpDrawArea.d_top += thisCount * getLineSpacing(y_scale);
 			break;
 
+		case WordWrapJustified:
+			// no change needed
+			thisCount = drawWrappedText(currLine, tmpDrawArea, z, clip_rect, Justified, colours, x_scale, y_scale);
+			tmpDrawArea.d_top += thisCount * getLineSpacing(y_scale);
+			break;
+
 		default:
 			throw InvalidRequestException((utf8*)"Font::drawText - Unknown or unsupported TextFormatting value specified.");
 		}
@@ -295,7 +336,8 @@ uint Font::drawText(const String& text, const Rect& draw_area, float z, const Re
 
 	}
 
-	return lineCount;
+    // should not return 0
+    return ceguimax(lineCount, (size_t)1);
 }
 
 
@@ -325,12 +367,13 @@ uint Font::getRequiredTextureSize(const String& glyph_set)
 	uint	cur_x = 0;
 	uint	cur_y = d_maxGlyphHeight;
 
-	uint	glyph_set_length = glyph_set.length();
+	uint	glyph_set_length = static_cast<uint>(glyph_set.length());
 
 	for (uint i = 0; i < glyph_set_length; ++i)
 	{
 		// load-up required glyph
-		if (FT_Load_Char(d_impldat->fontFace, glyph_set[i], FT_LOAD_RENDER|(d_antiAliased ? 0 : FT_LOAD_MONOCHROME)))
+		if (FT_Load_Char(d_impldat->fontFace, glyph_set[i], FT_LOAD_RENDER|FT_LOAD_FORCE_AUTOHINT|
+            (d_antiAliased ? FT_LOAD_TARGET_NORMAL : FT_LOAD_MONOCHROME | FT_LOAD_TARGET_MONO)))
 		{
 			// skip errors
 			continue;
@@ -373,7 +416,7 @@ uint Font::getRequiredTextureSize(const String& glyph_set)
 	Render a set of glyph images into the given memory buffer.
 	pixels will be in A8R8G8B8 format
 *************************************************************************/
-void Font::createFontGlyphSet(const String& glyph_set, uint size, uint32* buffer)
+void Font::createFontGlyphSet(const String& glyph_set, uint size, argb_t* buffer)
 {
 	String	imageName;
 	Rect	rect;
@@ -383,7 +426,7 @@ void Font::createFontGlyphSet(const String& glyph_set, uint size, uint32* buffer
 
 	d_max_bearingY = 0;
 
-	uint	glyph_set_length = glyph_set.length();
+	size_t	glyph_set_length = glyph_set.length();
 	uint	cur_x = 0;
 	uint	cur_y = 0;
 	uint	width;
@@ -391,7 +434,8 @@ void Font::createFontGlyphSet(const String& glyph_set, uint size, uint32* buffer
 	for (uint i = 0; i < glyph_set_length; ++i)
 	{
 		// load-up required glyph
-		if (FT_Load_Char(d_impldat->fontFace, glyph_set[i], FT_LOAD_RENDER|(d_antiAliased ? 0 : FT_LOAD_MONOCHROME)))
+		if (FT_Load_Char(d_impldat->fontFace, glyph_set[i], FT_LOAD_RENDER|FT_LOAD_FORCE_AUTOHINT|
+            (d_antiAliased ? FT_LOAD_TARGET_NORMAL : FT_LOAD_MONOCHROME | FT_LOAD_TARGET_MONO)))
 		{
 			// skip errors (but now we log them!)
 			std::stringstream err;
@@ -412,7 +456,7 @@ void Font::createFontGlyphSet(const String& glyph_set, uint size, uint32* buffer
 		}
 
 		// calculate offset into buffer for this glyph
-		uint32* dest_buff = buffer + (cur_y * size) + cur_x;
+		argb_t* dest_buff = buffer + (cur_y * size) + cur_x;
 
 		// draw glyph into buffer
 		drawGlyphToBuffer(dest_buff, size);
@@ -423,6 +467,7 @@ void Font::createFontGlyphSet(const String& glyph_set, uint size, uint32* buffer
 		rect.d_top		= (float)cur_y;
 		rect.d_right	= (float)(cur_x + width - InterGlyphPadSpace);
 		rect.d_bottom	= (float)(cur_y + d_maxGlyphHeight - InterGlyphPadSpace);
+
 		offset.d_x		= (float)(glyph->metrics.horiBearingX >> 6);
 		offset.d_y		= -(float)(glyph->metrics.horiBearingY >> 6);
 
@@ -450,7 +495,7 @@ void Font::createFontGlyphSet(const String& glyph_set, uint size, uint32* buffer
 	Render a range of glyph images into the given memory buffer.
 	pixels will be in A8R8G8B8 format
 *************************************************************************/
-void Font::createFontGlyphSet(utf32 first_code_point, utf32 last_code_point, uint size, uint32* buffer)
+void Font::createFontGlyphSet(utf32 first_code_point, utf32 last_code_point, uint size, argb_t* buffer)
 {
 	String tmp;
 
@@ -500,7 +545,7 @@ void Font::defineFontGlyphs(utf32 first_code_point, utf32 last_code_point)
 /*************************************************************************
 	Copy the FreeType glyph bitmap into the given memory buffer
 *************************************************************************/
-void Font::drawGlyphToBuffer(uint32* buffer, uint buf_width)
+void Font::drawGlyphToBuffer(argb_t* buffer, uint buf_width)
 {
 	FT_Bitmap* glyph_bitmap = &d_impldat->fontFace->glyph->bitmap;
 
@@ -540,15 +585,15 @@ void Font::drawGlyphToBuffer(uint32* buffer, uint buf_width)
 /*************************************************************************
 	draws wrapped text
 *************************************************************************/
-uint Font::drawWrappedText(const String& text, const Rect& draw_area, float z, const Rect& clip_rect, TextFormatting fmt, const ColourRect& colours, float x_scale, float y_scale) const
+size_t Font::drawWrappedText(const String& text, const Rect& draw_area, float z, const Rect& clip_rect, TextFormatting fmt, const ColourRect& colours, float x_scale, float y_scale) const
 {
-	uint	line_count = 0;
+	size_t	line_count = 0;
 	Rect	dest_area(draw_area);
 	float	wrap_width = draw_area.getWidth();
 
 	String  whitespace = TextUtils::DefaultWhitespace;
 	String	thisLine, thisWord;
-	uint	currpos = 0;
+	size_t	currpos = 0;
 
 	// get first word.
 	currpos += getNextWord(text, currpos, thisLine);
@@ -577,8 +622,10 @@ uint Font::drawWrappedText(const String& text, const Rect& draw_area, float z, c
 		thisLine += thisWord;
 	}
 
+	// Last line is left aligned
+	TextFormatting last_fmt = (fmt == Justified ? LeftAligned : fmt);
 	// output last bit of string
-	line_count += drawText(thisLine, dest_area, z, clip_rect, fmt, colours, x_scale, y_scale);
+	line_count += drawText(thisLine, dest_area, z, clip_rect, last_fmt, colours, x_scale, y_scale);
 
 	return line_count;
 }
@@ -587,7 +634,7 @@ uint Font::drawWrappedText(const String& text, const Rect& draw_area, float z, c
 /*************************************************************************
 	helper function for renderWrappedText to get next word of a string
 *************************************************************************/
-uint Font::getNextWord(const String& in_string, uint start_idx, String& out_string) const
+size_t Font::getNextWord(const String& in_string, size_t start_idx, String& out_string) const
 {
 	out_string = TextUtils::getNextWord(in_string, start_idx, TextUtils::DefaultWrapDelimiters);
 
@@ -604,10 +651,10 @@ void Font::drawTextLine(const String& text, const Vector3& position, const Rect&
 
 	float base_y = position.d_y;
 
-	uint char_count = text.length();
+	size_t char_count = text.length();
 	CodepointMap::const_iterator	pos, end = d_cp_map.end();
 
-	for (uint c = 0; c < char_count; ++c)
+	for (size_t c = 0; c < char_count; ++c)
 	{
 		pos = d_cp_map.find(text[c]);
 
@@ -625,12 +672,56 @@ void Font::drawTextLine(const String& text, const Vector3& position, const Rect&
 }
 
 
+/*************************************************************************
+	Draw a justified line of text.
+*************************************************************************/
+void Font::drawTextLineJustified(const String& text, const Rect& draw_area, const Vector3& position, const Rect& clip_rect, const ColourRect& colours, float x_scale, float y_scale) const
+{
+	Vector3	cur_pos(position);
+
+	float base_y = position.d_y;
+	// Calculate the length difference between the justified text and the same text, left aligned
+	// This space has to be shared between the space characters of the line
+	float lost_space = getFormattedTextExtent(text, draw_area, Justified, x_scale) - getTextExtent(text, x_scale);
+
+	size_t char_count = text.length();
+	CodepointMap::const_iterator	pos, end = d_cp_map.end();
+
+	// The number of spaces and tabs in the current line
+	uint space_count = 0;
+    size_t c;
+	for (c = 0; c < char_count; ++c)
+		if ((text[c] == ' ') || (text[c] == '\t')) ++space_count;
+
+	// The width that must be added to each space character in order to transform the left aligned text in justified text
+	float shared_lost_space = 0.0;
+	if (space_count > 0) shared_lost_space = lost_space / (float)space_count;
+
+	for (c = 0; c < char_count; ++c)
+	{
+		pos = d_cp_map.find(text[c]);
+
+		if (pos != end)
+		{
+			const Image* img = pos->second.d_image;
+			cur_pos.d_y = base_y - (img->getOffsetY() - img->getOffsetY() * y_scale);
+			Size sz(img->getWidth() * x_scale, img->getHeight() * y_scale);
+			img->draw(cur_pos, sz, clip_rect, colours);
+			cur_pos.d_x += (float)pos->second.d_horz_advance * x_scale;
+			// That's where we adjust the size of each space character
+			if ((text[c] == ' ') || (text[c] == '\t')) cur_pos.d_x += shared_lost_space;
+		}
+
+	}
+
+}
+
+
 /*************************************************************************
 	Function to do real work of constructor
 *************************************************************************/
 void Font::constructor_impl(const String& name, const String& fontname, const String& resourceGroup, uint size, uint flags, const String& glyph_set)
 {
-	FontManager&	 fman	= FontManager::getSingleton();
 	ImagesetManager& ismgr	= ImagesetManager::getSingleton();
 
 	// pull a-a setting from flags
@@ -701,8 +792,6 @@ void Font::constructor_impl(const String& name, const String& fontname, const St
 *************************************************************************/
 void Font::load(const String& filename, const String& resourceGroup)
 {
-	XERCES_CPP_NAMESPACE_USE
-
 	// unload old data
 	unload();
 
@@ -711,86 +800,22 @@ void Font::load(const String& filename, const String& resourceGroup)
 		throw InvalidRequestException((utf8*)"Font::load - Filename supplied for Font loading must be valid");
 	}
 
-	SAX2XMLReader* parser = XMLReaderFactory::createXMLReader();
-
-	// set basic settings we want from parser
-	parser->setFeature(XMLUni::fgSAX2CoreValidation, true);
-	parser->setFeature(XMLUni::fgSAX2CoreNameSpaces, true);
-	parser->setFeature(XMLUni::fgXercesSchema, true);
-	parser->setFeature(XMLUni::fgXercesValidationErrorAsFatal, true);
-
-//    InputSourceContainer fontSchemaData;
-//    System::getSingleton().getResourceProvider()->loadInputSourceContainer(FontSchemaName, fontSchemaData);
-//    parser->loadGrammar(*(fontSchemaData.getDataPtr()), Grammar::SchemaGrammarType, true);
-
-    RawDataContainer rawSchemaData;
-    System::getSingleton().getResourceProvider()->loadRawDataContainer(FontSchemaName, rawSchemaData, resourceGroup);
-    MemBufInputSource  fontSchemaData(rawSchemaData.getDataPtr(), rawSchemaData.getSize(), FontSchemaName, false);
-    parser->loadGrammar(fontSchemaData, Grammar::SchemaGrammarType, true);
-    // enable grammar reuse
-    parser->setFeature(XMLUni::fgXercesUseCachedGrammarInParse, true);
-
-	// setup schema for Font data
-	XMLCh* pval = XMLString::transcode(FontSchemaName);
-	parser->setProperty(XMLUni::fgXercesSchemaExternalNoNameSpaceSchemaLocation, pval);
-	XMLString::release(&pval);
-
-	// setup handler object
-	Font_xmlHandler handler(this);
-	parser->setContentHandler(&handler);
-	parser->setErrorHandler(&handler);
-
-//    InputSourceContainer fontData;
-//    System::getSingleton().getResourceProvider()->loadInputSourceContainer(filename, fontData);
-
-    RawDataContainer rawXMLData;
-    System::getSingleton().getResourceProvider()->loadRawDataContainer(filename, rawXMLData, resourceGroup);
-    MemBufInputSource  fontData(rawXMLData.getDataPtr(), rawXMLData.getSize(), filename.c_str(), false);
+    // create handler object
+    Font_xmlHandler handler(this);
 
 	// do parse (which uses handler to create actual data)
 	try
 	{
-//        parser->parse(*(fontData.getDataPtr()));
-        parser->parse(fontData);
-	}
-	catch(const XMLException& exc)
-	{
-		if (exc.getCode() != XMLExcepts::NoError)
-		{
-			unload();
-			delete parser;
-
-			char* excmsg = XMLString::transcode(exc.getMessage());
-			String message((utf8*)"Font::load - An error occurred while parsing Font file '" + filename + "'.  Additional information: ");
-			message += excmsg;
-			XMLString::release(&excmsg);
-
-			throw FileIOException(message);
-		}
-
-	}
-	catch(const SAXParseException& exc)
-	{
-		unload();
-		delete parser;
-
-		char* excmsg = XMLString::transcode(exc.getMessage());
-		String message((utf8*)"Font::load - An error occurred while parsing Font file '" + filename + "'.  Additional information: ");
-		message += excmsg;
-		XMLString::release(&excmsg);
-
-		throw FileIOException(message);
+        System::getSingleton().getXMLParser()->parseXMLFile(handler, filename, FontSchemaName, resourceGroup);
 	}
 	catch(...)
 	{
 		unload();
-		delete parser;
 
-		throw FileIOException((utf8*)"Font::load - An unexpected error occurred while parsing Font file '" + filename + "'.");
+        Logger::getSingleton().logEvent("Font::load - loading of Font from file '" + filename +"' failed.", Errors);
+		throw;
 	}
 
-	// cleanup
-	delete parser;
 }
 
 
@@ -816,6 +841,8 @@ void Font::unload(void)
 		d_freetype = false;
 	}
 
+	System::getSingleton().getResourceProvider()->unloadRawDataContainer(d_impldat->fontData);
+
 }
 
 
@@ -839,11 +866,11 @@ void Font::defineFontGlyphs_impl(void)
 	}
 
 	// allocate memory buffer where we will define the imagery
-	uint32* mem_buffer;
+	argb_t* mem_buffer;
 
 	try
 	{
-		mem_buffer = new uint32[texture_size * texture_size];
+		mem_buffer = new argb_t[texture_size * texture_size];
 	}
 	catch (std::bad_alloc)
 	{
@@ -851,7 +878,7 @@ void Font::defineFontGlyphs_impl(void)
 	}
 
 	// initialise background to transparent black.
-	memset(mem_buffer, 0, ((texture_size * texture_size) * sizeof(uint32)));
+	memset(mem_buffer, 0, ((texture_size * texture_size) * sizeof(argb_t)));
 
 	// clear old data about glyphs and images
 	d_cp_map.clear();
@@ -891,8 +918,8 @@ void Font::calculateStaticVertSpacing(void)
 		{
 			const Image* img = pos->second.d_image;
 
-			if (img->getOffsetY() > d_max_bearingY)
-				d_max_bearingY = img->getOffsetY();
+			if (-img->getOffsetY() > d_max_bearingY)
+				d_max_bearingY = -img->getOffsetY();
 
 			if (img->getHeight() > d_lineHeight)
 				d_lineHeight = img->getHeight();
@@ -901,6 +928,7 @@ void Font::calculateStaticVertSpacing(void)
 		d_max_bearingY *= scale;
 		d_lineHeight *= scale;
 		d_lineSpacing = d_lineHeight;
+        d_maxGlyphHeight = static_cast<uint>(d_lineHeight);
 	}
 
 }
@@ -1028,22 +1056,22 @@ void Font::createFontFromFT_Face(uint size, uint horzDpi, uint vertDpi)
 /*************************************************************************
 	Return the number of lines the given text would be formatted to.	
 *************************************************************************/
-uint Font::getFormattedLineCount(const String& text, const Rect& format_area, TextFormatting fmt, float x_scale) const
+size_t Font::getFormattedLineCount(const String& text, const Rect& format_area, TextFormatting fmt, float x_scale) const
 {
 	// handle simple non-wrapped cases.
-	if ((fmt == LeftAligned) || (fmt == Centred) || (fmt == RightAligned))
+	if ((fmt == LeftAligned) || (fmt == Centred) || (fmt == RightAligned) || (fmt == Justified))
 	{
-		return std::count(text.begin(), text.end(), '\n') + 1;
+		return std::count(text.begin(), text.end(), static_cast<utf8>('\n')) + 1;
 	}
 
 	// handle wraping cases
-	uint lineStart = 0, lineEnd = 0;
+	size_t lineStart = 0, lineEnd = 0;
 	String	sourceLine;
 
 	float	wrap_width = format_area.getWidth();
 	String  whitespace = TextUtils::DefaultWhitespace;
 	String	thisLine, thisWord;
-	uint	line_count = 0, currpos = 0;
+	size_t	line_count = 0, currpos = 0;
 
 	while (lineEnd < text.length())
 	{
@@ -1122,7 +1150,7 @@ float Font::getFormattedTextExtent(const String& text, const Rect& format_area,
 	float lineWidth;
 	float widest = 0;
 
-	uint lineStart = 0, lineEnd = 0;
+	size_t lineStart = 0, lineEnd = 0;
 	String	currLine;
 
 	while (lineEnd < text.length())
@@ -1143,12 +1171,22 @@ float Font::getFormattedTextExtent(const String& text, const Rect& format_area,
 			lineWidth = getTextExtent(currLine, x_scale);
 			break;
 
+		case Justified:
+			// usually we use the width of the rect but we have to ensure the current line is not wider than that
+			lineWidth = ceguimax(format_area.getWidth(), getTextExtent(currLine, x_scale));
+			break;
+
 		case WordWrapLeftAligned:
 		case WordWrapRightAligned:
 		case WordWrapCentred:
 			lineWidth = getWrappedTextExtent(currLine, format_area.getWidth(), x_scale);
 			break;
 
+		case WordWrapJustified:
+			// same as above
+			lineWidth = ceguimax(format_area.getWidth(), getWrappedTextExtent(currLine, format_area.getWidth(), x_scale));
+			break;
+
 		default:
 			throw InvalidRequestException((utf8*)"Font::getFormattedTextExtent - Unknown or unsupported TextFormatting value specified.");
 		}
@@ -1171,7 +1209,7 @@ float Font::getWrappedTextExtent(const String& text, float wrapWidth, float x_sc
 {
 	String  whitespace = TextUtils::DefaultWhitespace;
 	String	thisWord;
-	uint	currpos;
+	size_t	currpos;
 	float	lineWidth, wordWidth;
 	float	widest = 0;
 
@@ -1239,4 +1277,67 @@ uint Font::getPointSize(void) const
 	}
 }
 
+
+/*************************************************************************
+    Writes an xml representation of this Font to \a out_stream.
+*************************************************************************/
+void Font::writeXMLToStream(OutStream& out_stream) const
+{
+    // output starting <Font ... > element
+    out_stream << "<Font Name=\"" << d_name << "\" Filename=\"" << d_sourceFilename << "\" ";
+
+    if (d_freetype)
+        out_stream << "Size=\"" << d_ptSize << "\" ";
+
+    if (d_nativeHorzRes != DefaultNativeHorzRes)
+        out_stream << "NativeHorzRes=\"" << static_cast<uint>(d_nativeHorzRes) << "\" ";
+
+    if (d_nativeVertRes != DefaultNativeVertRes)
+        out_stream << "NativeVertRes=\"" << static_cast<uint>(d_nativeVertRes) << "\" ";
+
+    if (d_autoScale)
+        out_stream << "AutoScaled=\"True\" ";
+
+    out_stream << ">" << std::endl;
+
+    // dynamic font so output defined glyphs
+    if (d_freetype)
+    {
+        size_t start = 0, idx = 0;
+
+        while(start < d_glyphset.length())
+        {
+            // find end of range
+            while ((idx + 1 < d_glyphset.length()) && (d_glyphset[idx] + 1 == d_glyphset[idx + 1]))
+                ++idx;
+
+            if (start == idx)
+                // if range is a just a single codepoint
+                out_stream << "<Glyph Codepoint=\"" << d_glyphset[start] << "\" />" << std::endl;
+            else
+                // range contains >1 codepoint
+                out_stream << "<GlyphRange StartCodepoint=\"" << d_glyphset[start] << "\" EndCodepoint=\"" << d_glyphset[idx] << "\" />" << std::endl;
+
+            start = ++idx;
+        }
+    }
+    // static font, so output glyph to imageset mappings
+    else
+    {
+        for (CodepointMap::const_iterator iter = d_cp_map.begin(); iter != d_cp_map.end(); ++iter)
+        {
+            out_stream << "<Mapping Codepoint=\"" << (*iter).first << "\" Image=\"" << (*iter).second.d_image->getName() << "\" ";
+
+            if ((*iter).second.d_horz_advance_unscaled != -1)
+                out_stream << "HorzAdvance=\"" << (*iter).second.d_horz_advance_unscaled << "\" ";
+
+            out_stream << "/>" << std::endl;
+        }
+    }
+
+    // output closing </Font> element.
+    out_stream << "</Font>" << std::endl;
+}
+
+
 } // End of  CEGUI namespace section
