@@ -24,9 +24,11 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *************************************************************************/
 #include "CEGUIExceptions.h"
+#include "CEGUIWindowManager.h"
 #include "elements/CEGUIListbox.h"
 #include "elements/CEGUIListboxItem.h"
 #include "elements/CEGUIScrollbar.h"
+#include "elements/CEGUITooltip.h"
 
 #include <algorithm>
 
@@ -42,6 +44,7 @@ ListboxProperties::Sort					Listbox::d_sortProperty;
 ListboxProperties::MultiSelect			Listbox::d_multiSelectProperty;
 ListboxProperties::ForceVertScrollbar	Listbox::d_forceVertProperty;
 ListboxProperties::ForceHorzScrollbar	Listbox::d_forceHorzProperty;
+ListboxProperties::ItemTooltips			Listbox::d_itemTooltipsProperty;
 
 
 /*************************************************************************
@@ -65,6 +68,7 @@ Listbox::Listbox(const String& type, const String& name)
 	d_multiselect(false),
 	d_forceVertScroll(false),
 	d_forceHorzScroll(false),
+	d_itemTooltips(false),
 	d_lastSelected(NULL)
 {
 	// add new events specific to list box.
@@ -89,25 +93,28 @@ Listbox::~Listbox(void)
 void Listbox::initialise(void)
 {
 	// create the component sub-widgets
-	d_vertScrollbar = createVertScrollbar();
-	d_horzScrollbar = createHorzScrollbar();
+	d_vertScrollbar = createVertScrollbar(getName() + "__auto_vscrollbar__");
+	d_horzScrollbar = createHorzScrollbar(getName() + "__auto_hscrollbar__");
 
 	addChildWindow(d_vertScrollbar);
 	addChildWindow(d_horzScrollbar);
 
+    d_vertScrollbar->subscribeEvent(Scrollbar::EventScrollPositionChanged, Event::Subscriber(&Listbox::handle_scrollChange, this));
+    d_horzScrollbar->subscribeEvent(Scrollbar::EventScrollPositionChanged, Event::Subscriber(&Listbox::handle_scrollChange, this));
+
 	configureScrollbars();
-	layoutComponentWidgets();
+	performChildWindowLayout();
 }
 
 
 /*************************************************************************
 	Return the number of selected items in the list box.	
 *************************************************************************/
-uint Listbox::getSelectedCount(void) const
+size_t Listbox::getSelectedCount(void) const
 {
-	uint count = 0;
+	size_t count = 0;
 
-	for (uint index = 0; index < (uint)d_listItems.size(); ++index)
+	for (size_t index = 0; index < d_listItems.size(); ++index)
 	{
 		if (d_listItems[index]->isSelected())
 		{
@@ -135,9 +142,9 @@ ListboxItem* Listbox::getFirstSelectedItem(void) const
 ListboxItem* Listbox::getNextSelected(const ListboxItem* start_item) const
 {
 	// if start_item is NULL begin search at begining, else start at item after start_item
-	uint index = (start_item == NULL) ? 0 : (getItemIndex(start_item) + 1);
+	size_t index = (start_item == NULL) ? 0 : (getItemIndex(start_item) + 1);
 
-	while (index < (uint)d_listItems.size())
+	while (index < d_listItems.size())
 	{
 		// return pointer to this item if it's selected.
 		if (d_listItems[index]->isSelected())
@@ -160,9 +167,9 @@ ListboxItem* Listbox::getNextSelected(const ListboxItem* start_item) const
 /*************************************************************************
 	Return the item at index position 'index'.
 *************************************************************************/
-ListboxItem* Listbox::getListboxItemFromIndex(uint index) const
+ListboxItem* Listbox::getListboxItemFromIndex(size_t index) const
 {
-	if (index < (uint)d_listItems.size())
+	if (index < d_listItems.size())
 	{
 		return d_listItems[index];
 	}
@@ -176,13 +183,13 @@ ListboxItem* Listbox::getListboxItemFromIndex(uint index) const
 /*************************************************************************
 	Return the index of ListboxItem \a item
 *************************************************************************/
-uint Listbox::getItemIndex(const ListboxItem* item) const
+size_t Listbox::getItemIndex(const ListboxItem* item) const
 {
 	LBItemList::const_iterator pos = std::find(d_listItems.begin(), d_listItems.end(), item);
 
 	if (pos != d_listItems.end())
 	{
-		return (uint)std::distance(d_listItems.begin(), pos);
+		return std::distance(d_listItems.begin(), pos);
 	}
 	else
 	{
@@ -195,9 +202,9 @@ uint Listbox::getItemIndex(const ListboxItem* item) const
 /*************************************************************************
 	return whether the string at index position \a index is selected
 *************************************************************************/
-bool Listbox::isItemSelected(uint index) const
+bool Listbox::isItemSelected(size_t index) const
 {
-	if (index < (uint)d_listItems.size())
+	if (index < d_listItems.size())
 	{
 		return d_listItems[index]->isSelected();
 	}
@@ -215,9 +222,9 @@ bool Listbox::isItemSelected(uint index) const
 ListboxItem* Listbox::findItemWithText(const String& text, const ListboxItem* start_item)
 {
 	// if start_item is NULL begin search at begining, else start at item after start_item
-	uint index = (start_item == NULL) ? 0 : (getItemIndex(start_item) + 1);
+	size_t index = (start_item == NULL) ? 0 : (getItemIndex(start_item) + 1);
 
-	while (index < (uint)d_listItems.size())
+	while (index < d_listItems.size())
 	{
 		// return pointer to this item if it's text matches
 		if (d_listItems[index]->getText() == text)
@@ -429,7 +436,7 @@ void Listbox::setMultiselectEnabled(bool setting)
 		{
 			ListboxItem* itm = getFirstSelectedItem();
 
-			while (itm = getNextSelected(itm))
+			while ((itm = getNextSelected(itm)))
 			{
 				itm->setSelected(false);
 			}
@@ -443,6 +450,12 @@ void Listbox::setMultiselectEnabled(bool setting)
 
 }
 
+void Listbox::setItemTooltipsEnabled(bool setting)
+{
+	d_itemTooltips = setting;
+}
+
+
 
 /*************************************************************************
 	Set whether the vertical scroll bar should always be shown.
@@ -487,7 +500,7 @@ void Listbox::setItemSelectState(ListboxItem* item, bool state)
 
 	if (pos != d_listItems.end())
 	{
-		setItemSelectState((uint)std::distance(d_listItems.begin(), pos), state);
+		setItemSelectState(std::distance(d_listItems.begin(), pos), state);
 	}
 	else
 	{
@@ -499,7 +512,7 @@ void Listbox::setItemSelectState(ListboxItem* item, bool state)
 /*************************************************************************
 	Set the select state of an attached ListboxItem.	
 *************************************************************************/
-void Listbox::setItemSelectState(uint item_index, bool state)
+void Listbox::setItemSelectState(size_t item_index, bool state)
 {
 	if (item_index < getItemCount())
 	{
@@ -540,62 +553,58 @@ void Listbox::handleUpdatedItemData(void)
 /*************************************************************************
 	Perform the actual rendering for this Window.
 *************************************************************************/
-void Listbox::drawSelf(float z)
+void Listbox::populateRenderCache()
 {
-	// get the derived class to render general stuff before we handle the items
-	renderListboxBaseImagery(z);
+    // get the derived class to render general stuff before we handle the items
+    cacheListboxBaseImagery();
 
-	//
-	// Render list items
-	//
-	Vector3	itemPos;
-	Size	itemSize;
-	Rect	itemClipper;
-	float	widest = getWidestItemWidth();
+    //
+    // Render list items
+    //
+    Vector3	itemPos;
+    Size	itemSize;
+    Rect	itemClipper, itemRect;
+    float	widest = getWidestItemWidth();
 
-	// calculate on-screen position of area we have to render into
-	Rect absarea(getListRenderArea());
-	absarea.offset(getUnclippedPixelRect().getPosition());
+    // calculate position of area we have to render into
+    Rect itemsArea(getListRenderArea());
 
-	// calculate clipper for list rendering area
-	Rect clipper(absarea.getIntersection(getPixelRect()));
+    // set up some initial positional details for items
+    itemPos.d_x = itemsArea.d_left - d_horzScrollbar->getScrollPosition();
+    itemPos.d_y = itemsArea.d_top - d_vertScrollbar->getScrollPosition();
+    itemPos.d_z = System::getSingleton().getRenderer()->getZLayer(3) - System::getSingleton().getRenderer()->getCurrentZ();
 
-	// set up some initial positional details for items
-	itemPos.d_x = PixelAligned(absarea.d_left - d_horzScrollbar->getScrollPosition());
-	itemPos.d_y = PixelAligned(absarea.d_top - d_vertScrollbar->getScrollPosition());
-	itemPos.d_z = System::getSingleton().getRenderer()->getZLayer(3);
+    float alpha = getEffectiveAlpha();
 
-	float alpha = getEffectiveAlpha();
+    // loop through the items
+    size_t itemCount = getItemCount();
 
-	// loop through the items
-	uint itemCount = getItemCount();
+    for (size_t i = 0; i < itemCount; ++i)
+    {
+        itemSize.d_height = d_listItems[i]->getPixelSize().d_height;
 
-	for (uint i = 0; i < itemCount; ++i)
-	{
-		itemSize.d_height = d_listItems[i]->getPixelSize().d_height;
+        // allow item to have full width of box if this is wider than items
+        itemSize.d_width = ceguimax(itemsArea.getWidth(), widest);
 
-		// allow item to have full width of box if this is wider than items
-		itemSize.d_width = ceguimax(absarea.getWidth(), widest);
+        // calculate destination area for this item.
+        itemRect.d_left	= itemPos.d_x;
+        itemRect.d_top	= itemPos.d_y;
+        itemRect.setSize(itemSize);
+        itemClipper = itemRect.getIntersection(itemsArea);
 
-		// calculate clipper for this item.
-		itemClipper.d_left	= itemPos.d_x;
-		itemClipper.d_top	= itemPos.d_y;
-		itemClipper.setSize(itemSize);
-		itemClipper = itemClipper.getIntersection(clipper);
+        // skip this item if totally clipped
+        if (itemClipper.getWidth() == 0)
+        {
+            itemPos.d_y += itemSize.d_height;
+            continue;
+        }
 
-		// skip this item if totally clipped
-		if (itemClipper.getWidth() == 0)
-		{
-			itemPos.d_y += itemSize.d_height;
-			continue;
-		}
+        // draw this item
+        d_listItems[i]->draw(d_renderCache, itemRect, itemPos.d_z, alpha, &itemClipper);
 
-		// draw this item
-		d_listItems[i]->draw(itemPos, alpha, itemClipper);
-
-		// update position ready for next item
-		itemPos.d_y += PixelAligned(itemSize.d_height);
-	}
+        // update position ready for next item
+        itemPos.d_y += itemSize.d_height;
+    }
 
 }
 
@@ -606,6 +615,20 @@ void Listbox::drawSelf(float z)
 *************************************************************************/
 void Listbox::configureScrollbars(void)
 {
+    Scrollbar* vertScrollbar;
+    Scrollbar* horzScrollbar;
+
+    try
+    {
+        vertScrollbar = static_cast<Scrollbar*>(WindowManager::getSingleton().getWindow(getName() + "__auto_vscrollbar__"));
+        horzScrollbar = static_cast<Scrollbar*>(WindowManager::getSingleton().getWindow(getName() + "__auto_hscrollbar__"));
+    }
+    catch (UnknownObjectException)
+    {
+        // no scrollbars?  Can't configure then!
+        return;
+    }
+
 	float totalHeight	= getTotalItemsHeight();
 	float widestItem	= getWidestItemWidth();
 
@@ -615,16 +638,16 @@ void Listbox::configureScrollbars(void)
 	// show or hide vertical scroll bar as required (or as specified by option)
 	if ((totalHeight > getListRenderArea().getHeight()) || d_forceVertScroll)
 	{
-		d_vertScrollbar->show();
+		vertScrollbar->show();
 
 		// show or hide horizontal scroll bar as required (or as specified by option)
 		if ((widestItem > getListRenderArea().getWidth()) || d_forceHorzScroll)
 		{
-			d_horzScrollbar->show();
+			horzScrollbar->show();
 		}
 		else
 		{
-			d_horzScrollbar->hide();
+			horzScrollbar->hide();
 		}
 
 	}
@@ -633,23 +656,23 @@ void Listbox::configureScrollbars(void)
 		// show or hide horizontal scroll bar as required (or as specified by option)
 		if ((widestItem > getListRenderArea().getWidth()) || d_forceHorzScroll)
 		{
-			d_horzScrollbar->show();
+			horzScrollbar->show();
 
 			// show or hide vertical scroll bar as required (or as specified by option)
 			if ((totalHeight > getListRenderArea().getHeight()) || d_forceVertScroll)
 			{
-				d_vertScrollbar->show();
+				vertScrollbar->show();
 			}
 			else
 			{
-				d_vertScrollbar->hide();
+				vertScrollbar->hide();
 			}
 
 		}
 		else
 		{
-			d_vertScrollbar->hide();
-			d_horzScrollbar->hide();
+			vertScrollbar->hide();
+			horzScrollbar->hide();
 		}
 
 	}
@@ -659,22 +682,22 @@ void Listbox::configureScrollbars(void)
 	//
 	Rect renderArea(getListRenderArea());
 
-	d_vertScrollbar->setDocumentSize(totalHeight);
-	d_vertScrollbar->setPageSize(renderArea.getHeight());
-	d_vertScrollbar->setStepSize(ceguimax(1.0f, renderArea.getHeight() / 10.0f));
-	d_vertScrollbar->setScrollPosition(d_vertScrollbar->getScrollPosition());
+	vertScrollbar->setDocumentSize(totalHeight);
+	vertScrollbar->setPageSize(renderArea.getHeight());
+	vertScrollbar->setStepSize(ceguimax(1.0f, renderArea.getHeight() / 10.0f));
+	vertScrollbar->setScrollPosition(vertScrollbar->getScrollPosition());
 
-	d_horzScrollbar->setDocumentSize(widestItem);
-	d_horzScrollbar->setPageSize(renderArea.getWidth());
-	d_horzScrollbar->setStepSize(ceguimax(1.0f, renderArea.getWidth() / 10.0f));
-	d_horzScrollbar->setScrollPosition(d_horzScrollbar->getScrollPosition());
+	horzScrollbar->setDocumentSize(widestItem);
+	horzScrollbar->setPageSize(renderArea.getWidth());
+	horzScrollbar->setStepSize(ceguimax(1.0f, renderArea.getWidth() / 10.0f));
+	horzScrollbar->setScrollPosition(horzScrollbar->getScrollPosition());
 }
 
 
 /*************************************************************************
 	select all strings between positions 'start' and 'end' (inclusive)
 *************************************************************************/
-void Listbox::selectRange(uint start, uint end)
+void Listbox::selectRange(size_t start, size_t end)
 {
 	// only continue if list has some items
 	if (d_listItems.size() > 0)
@@ -694,7 +717,7 @@ void Listbox::selectRange(uint start, uint end)
 		// ensure start becomes before the end.
 		if (start > end)
 		{
-			uint tmp;
+			size_t tmp;
 			tmp = start;
 			start = end;
 			end = tmp;
@@ -718,7 +741,7 @@ float Listbox::getTotalItemsHeight(void) const
 {
 	float height = 0;
 
-	for (uint i = 0; i < getItemCount(); ++i)
+	for (size_t i = 0; i < getItemCount(); ++i)
 	{
 		height += d_listItems[i]->getPixelSize().d_height;
 	}
@@ -734,7 +757,7 @@ float Listbox::getWidestItemWidth(void) const
 {
 	float widest = 0;
 
-	for (uint i = 0; i < getItemCount(); ++i)
+	for (size_t i = 0; i < getItemCount(); ++i)
 	{
 		float thisWidth = d_listItems[i]->getPixelSize().d_width;
 
@@ -757,7 +780,7 @@ bool Listbox::clearAllSelections_impl(void)
 	// flag used so we can track if we did anything.
 	bool modified = false;
 
-	for (uint index = 0; index < (uint)d_listItems.size(); ++index)
+	for (size_t index = 0; index < d_listItems.size(); ++index)
 	{
 		if (d_listItems[index]->isSelected())
 		{
@@ -786,7 +809,7 @@ ListboxItem* Listbox::getItemAtPoint(const Point& pt) const
 		// test if point is above first item
 		if (pt.d_y >= y)
 		{
-			for (uint i = 0; i < getItemCount(); ++i)
+			for (size_t i = 0; i < getItemCount(); ++i)
 			{
 				y += d_listItems[i]->getPixelSize().d_height;
 
@@ -889,7 +912,6 @@ void Listbox::onSized(WindowEventArgs& e)
 	Window::onSized(e);
 
 	configureScrollbars();
-	layoutComponentWidgets();
 
 	e.handled = true;
 }
@@ -973,11 +995,51 @@ void Listbox::onMouseWheel(MouseEventArgs& e)
 	e.handled = true;
 }
 
+/*************************************************************************
+    Handler for mouse movement
+*************************************************************************/
+void Listbox::onMouseMove(MouseEventArgs& e)
+{
+    if (d_itemTooltips)
+    {
+        static ListboxItem* lastItem = NULL;
+
+        Point posi = relativeToAbsolute(screenToWindow(e.position));
+        ListboxItem* item = getItemAtPoint(posi);
+        if (item != lastItem)
+        {
+            if (item != NULL)
+            {
+                setTooltipText(item->getTooltipText());
+            }
+            else
+            {
+                setTooltipText("");
+            }
+            lastItem = item;
+        }
+
+        // must check the result from getTooltip(), as the tooltip object could
+        // be 0 at any time for various reasons.
+        Tooltip* tooltip = getTooltip();
+
+        if (tooltip)
+        {
+            if (tooltip->getTargetWindow() != this)
+                tooltip->setTargetWindow(this);
+            else
+                tooltip->positionSelf();
+        }
+    }
+
+    Window::onMouseMove(e);
+}
+
 
 /*************************************************************************
 	Ensure the item at the specified index is visible within the list box.	
 *************************************************************************/
-void Listbox::ensureItemIsVisible(uint item_index)
+void Listbox::ensureItemIsVisible(size_t item_index)
 {
 	// handle simple "scroll to the bottom" case
 	if (item_index >= getItemCount())
@@ -991,7 +1053,7 @@ void Listbox::ensureItemIsVisible(uint item_index)
 		float top = 0;
 
 		// get height to top of item
-		uint i;
+		size_t i;
 		for (i = 0; i < item_index; ++i)
 		{
 			top += d_listItems[i]->getPixelSize().d_height;
@@ -1059,6 +1121,7 @@ void Listbox::addListboxProperties(void)
 	addProperty(&d_multiSelectProperty);
 	addProperty(&d_forceHorzProperty);
 	addProperty(&d_forceVertProperty);
+	addProperty(&d_itemTooltipsProperty);
 }
 
 
@@ -1076,7 +1139,7 @@ bool Listbox::resetList_impl(void)
 	else
 	{
 		// delete any items we are supposed to
-		for (uint i = 0; i < getItemCount(); ++i)
+		for (size_t i = 0; i < getItemCount(); ++i)
 		{
 			// if item is supposed to be deleted by us
 			if (d_listItems[i]->isAutoDeleted())
@@ -1097,6 +1160,16 @@ bool Listbox::resetList_impl(void)
 
 }
 
+/*************************************************************************
+    Handler for scroll position changes.
+*************************************************************************/
+bool Listbox::handle_scrollChange(const EventArgs& args)
+{
+    // simply trigger a redraw of the Listbox.
+    requestRedraw();
+    return true;
+}
+
 
 //////////////////////////////////////////////////////////////////////////
 /*************************************************************************
