@@ -29,7 +29,7 @@
 #include "elements/CEGUIListHeader.h"
 #include "elements/CEGUIListboxItem.h"
 #include "CEGUILogger.h"
-
+#include "CEGUIPropertyHelper.h"
 #include <algorithm>
 
 
@@ -51,7 +51,8 @@ MultiColumnListProperties::SelectionMode				MultiColumnList::d_selectModePropert
 MultiColumnListProperties::SortColumnID					MultiColumnList::d_sortColumnIDProperty;
 MultiColumnListProperties::SortDirection				MultiColumnList::d_sortDirectionProperty;
 MultiColumnListProperties::SortSettingEnabled			MultiColumnList::d_sortSettingProperty;
-
+MultiColumnListProperties::ColumnHeader					MultiColumnList::d_columnHeaderProperty;
+MultiColumnListProperties::RowCount						MultiColumnList::d_rowCountProperty;
 
 /*************************************************************************
 	Constants
@@ -611,9 +612,9 @@ MultiColumnList::SelectionMode MultiColumnList::getSelectionMode(void) const
 void MultiColumnList::initialise(void)
 {
 	// create the component sub-widgets
-	d_vertScrollbar = createVertScrollbar();
-	d_horzScrollbar = createHorzScrollbar();
-	d_header		= createListHeader();
+	d_vertScrollbar = createVertScrollbar(getName() + "__auto_vscrollbar__");
+	d_horzScrollbar = createHorzScrollbar(getName() + "__auto_hscrollbar__");
+	d_header		= createListHeader(getName() + "__auto_listheader__");
 
 	// add components
 	addChildWindow(d_vertScrollbar);
@@ -628,13 +629,15 @@ void MultiColumnList::initialise(void)
 	d_header->subscribeEvent(ListHeader::EventSortDirectionChanged, Event::Subscriber(&CEGUI::MultiColumnList::handleSortDirectionChange, this));
 	d_header->subscribeEvent(ListHeader::EventSplitterDoubleClicked, Event::Subscriber(&CEGUI::MultiColumnList::handleHeaderSegDblClick, this));
 	d_horzScrollbar->subscribeEvent(Scrollbar::EventScrollPositionChanged, Event::Subscriber(&CEGUI::MultiColumnList::handleHorzScrollbar, this));
+    d_vertScrollbar->subscribeEvent(Scrollbar::EventScrollPositionChanged, Event::Subscriber(&CEGUI::MultiColumnList::handleVertScrollbar, this));
+
 
 	// final initialisation now widget is complete
 	setSortDirection(ListHeaderSegment::None);
 
 	// Perform initial layout
 	configureScrollbars();
-	layoutComponentWidgets();
+	performChildWindowLayout();
 }
 
 
@@ -776,16 +779,16 @@ void MultiColumnList::moveColumnWithID(uint col_id, uint position)
 /*************************************************************************
 	Add a row to the bottom of the table
 *************************************************************************/
-uint MultiColumnList::addRow(void)
+uint MultiColumnList::addRow(uint row_id)
 {
-	return addRow(NULL, 0);
+	return addRow(NULL, 0, row_id);
 }
 
 
 /*************************************************************************
 	Add a row to the bottom of the table
 *************************************************************************/
-uint MultiColumnList::addRow(ListboxItem* item, uint col_id)
+uint MultiColumnList::addRow(ListboxItem* item, uint col_id, uint row_id)
 {
 	uint col_idx = 0;
 
@@ -793,6 +796,7 @@ uint MultiColumnList::addRow(ListboxItem* item, uint col_id)
 	ListRow row;
 	row.d_sortColumn = getSortColumn();
 	row.d_items.resize(getColumnCount(), NULL);
+	row.d_rowID = row_id;
 
 	if (item != NULL)
 	{
@@ -809,8 +813,12 @@ uint MultiColumnList::addRow(ListboxItem* item, uint col_id)
 	// if sorting is enabled, insert at an appropriate position
 	if (getSortDirection() != ListHeaderSegment::None)
 	{
-		// insert item and get insert position index.
-		pos = (uint)std::distance(d_grid.begin(), d_grid.insert(std::upper_bound(d_grid.begin(), d_grid.end(), row), row));
+        // calculate where the row should be inserted
+        ListItemGrid::iterator ins_pos = std::upper_bound(d_grid.begin(), d_grid.end(), row);
+        // insert item and get final inserted position.
+        ListItemGrid::iterator final_pos = d_grid.insert(ins_pos, row);
+		// get final inserted position as an uint.
+		pos = (uint)std::distance(d_grid.begin(), final_pos);
 	}
 	// not sorted, just stick it on the end.
 	else
@@ -830,16 +838,16 @@ uint MultiColumnList::addRow(ListboxItem* item, uint col_id)
 /*************************************************************************
 	Insert a row into the table
 *************************************************************************/
-uint MultiColumnList::insertRow(uint row_idx)
+uint MultiColumnList::insertRow(uint row_idx, uint row_id)
 {
-	return insertRow(NULL, 0, row_idx);
+	return insertRow(NULL, 0, row_idx, row_id);
 }
 
 
 /*************************************************************************
 	Insert a row into the table
 *************************************************************************/
-uint MultiColumnList::insertRow(ListboxItem* item, uint col_id, uint row_idx)
+uint MultiColumnList::insertRow(ListboxItem* item, uint col_id, uint row_idx, uint row_id)
 {
 	// if sorting is enabled, use add instead of insert
 	if (getSortDirection() != ListHeaderSegment::None)
@@ -848,12 +856,11 @@ uint MultiColumnList::insertRow(ListboxItem* item, uint col_id, uint row_idx)
 	}
 	else
 	{
-		uint col_idx = 0;
-		
 		// Build the new row (empty)
 		ListRow row;
 		row.d_sortColumn = getSortColumn();
 		row.d_items.resize(getColumnCount(), NULL);
+		row.d_rowID = row_id;
 
 		// if row index is too big, just insert at end.
 		if (row_idx > getRowCount())
@@ -936,13 +943,15 @@ void MultiColumnList::setItem(ListboxItem* item, const MCLGridRef& position)
 	// delete old item as required
 	ListboxItem* oldItem = d_grid[position.row][position.column];
 
-	if ((oldItem != NULL) && item->isAutoDeleted())
+	if ((oldItem != NULL) && oldItem->isAutoDeleted())
 	{
 		delete oldItem;
 	}
 
 	// set new item.
-	item->setOwnerWindow(this);
+	if (item != NULL)
+		item->setOwnerWindow(this);
+
 	d_grid[position.row][position.column] = item;
 
 
@@ -1711,75 +1720,76 @@ void MultiColumnList::moveColumn_impl(uint col_idx, uint position)
 /*************************************************************************
 	renders the widget
 *************************************************************************/
-void MultiColumnList::drawSelf(float z)
+void MultiColumnList::populateRenderCache()
 {
-	// get the derived class to render general stuff before we handle the items
-	renderListboxBaseImagery(z);
+    // get the derived class to render general stuff before we handle the items
+    cacheListboxBaseImagery();
 
-	//
-	// Render list items
-	//
-	Vector3	itemPos;
-	Size	itemSize;
-	Rect	itemClipper;
+    //
+    // Render list items
+    //
+    Vector3	itemPos;
+    Size	itemSize;
+    Rect	itemClipper, itemRect;;
 
-	// calculate on-screen position of area we have to render into
-	Rect absarea(getListRenderArea());
-	absarea.offset(getUnclippedPixelRect().getPosition());
+    // calculate position of area we have to render into
+    Rect itemsArea(getListRenderArea());
+    
+    //absarea.offset(getUnclippedPixelRect().getPosition());
 
-	// calculate clipper for list rendering area
-	Rect clipper(absarea.getIntersection(getPixelRect()));
+// 	// calculate clipper for list rendering area
+// 	Rect clipper(absarea.getIntersection(getPixelRect()));
 
-	// set up initial positional details for items
-	itemPos.d_y = PixelAligned(absarea.d_top - d_vertScrollbar->getScrollPosition());
-	itemPos.d_z = System::getSingleton().getRenderer()->getZLayer(3);
+    // set up initial positional details for items
+    itemPos.d_y = itemsArea.d_top - d_vertScrollbar->getScrollPosition();
+    itemPos.d_z = System::getSingleton().getRenderer()->getZLayer(3) - System::getSingleton().getRenderer()->getCurrentZ();
 
-	float alpha = getEffectiveAlpha();
+    float alpha = getEffectiveAlpha();
 
-	// loop through the items
-	for (uint i = 0; i < getRowCount(); ++i)
-	{
-		// set initial x position for this row.
-		itemPos.d_x = PixelAligned(absarea.d_left - d_horzScrollbar->getScrollPosition());
+    // loop through the items
+    for (uint i = 0; i < getRowCount(); ++i)
+    {
+        // set initial x position for this row.
+        itemPos.d_x = itemsArea.d_left - d_horzScrollbar->getScrollPosition();
 
-		// calculate height for this row.
-		itemSize.d_height = getHighestRowItemHeight(i);
+        // calculate height for this row.
+        itemSize.d_height = getHighestRowItemHeight(i);
 
-		// loop through the columns in this row
-		for (uint j = 0; j < getColumnCount(); ++j)
-		{
-			// allow item to use full width of the column
-			itemSize.d_width = d_header->getColumnPixelWidth(j);
+        // loop through the columns in this row
+        for (uint j = 0; j < getColumnCount(); ++j)
+        {
+            // allow item to use full width of the column
+            itemSize.d_width = d_header->getColumnPixelWidth(j);
 
-			ListboxItem* item = d_grid[i][j];
+            ListboxItem* item = d_grid[i][j];
 
-			// is the item for this column set?
-			if (item != NULL)
-			{
-				// calculate clipper for this item.
-				itemClipper.d_left	= itemPos.d_x;
-				itemClipper.d_top	= itemPos.d_y;
-				itemClipper.setSize(itemSize);
-				itemClipper = itemClipper.getIntersection(clipper);
+            // is the item for this column set?
+            if (item)
+            {
+                // calculate destination area for this item.
+                itemRect.d_left	= itemPos.d_x;
+                itemRect.d_top	= itemPos.d_y;
+                itemRect.setSize(itemSize);
+                itemClipper = itemRect.getIntersection(itemsArea);
 
-				// skip this item if totally clipped
-				if (itemClipper.getWidth() == 0)
-				{
-					itemPos.d_x += PixelAligned(itemSize.d_width);
-					continue;
-				}
+                // skip this item if totally clipped
+                if (itemClipper.getWidth() == 0)
+                {
+                    itemPos.d_x += itemSize.d_width;
+                    continue;
+                }
 
-				// draw this item
-				item->draw(itemPos, alpha, itemClipper);
-			}
+                // draw this item
+                item->draw(d_renderCache, itemRect, itemPos.d_z, alpha, &itemClipper);
+            }
 
-			// update position for next column.
-			itemPos.d_x += PixelAligned(itemSize.d_width);
-		}
+            // update position for next column.
+            itemPos.d_x += itemSize.d_width;
+        }
 
-		// update position ready for next row
-		itemPos.d_y += PixelAligned(itemSize.d_height);
-	}
+        // update position ready for next row
+        itemPos.d_y += itemSize.d_height;
+    }
 
 }
 
@@ -1900,7 +1910,6 @@ void MultiColumnList::onSized(WindowEventArgs& e)
 	Window::onSized(e);
 
 	configureScrollbars();
-	layoutComponentWidgets();
 
 	e.handled = true;
 }
@@ -2035,7 +2044,16 @@ bool MultiColumnList::handleHorzScrollbar(const EventArgs& e)
 {
 	// set header offset to match scroll position
 	d_header->setSegmentOffset(d_header->absoluteToRelativeX(d_horzScrollbar->getScrollPosition()));
+    requestRedraw();
+	return true;
+}
 
+/*************************************************************************
+	Event handler for when vertical scroll bar is moved.
+*************************************************************************/
+bool MultiColumnList::handleVertScrollbar(const EventArgs& e)
+{
+    requestRedraw();
 	return true;
 }
 
@@ -2153,6 +2171,41 @@ uint MultiColumnList::getColumnID(uint col_idx) const
 }
 
 
+/*************************************************************************
+	Return the ID code assigned to the requested row.
+*************************************************************************/
+uint MultiColumnList::getRowID(uint row_idx) const
+{
+	// check for invalid index
+	if (row_idx >= getRowCount())
+	{
+		throw InvalidRequestException((utf8*)"MultiColumnList::getRowID - the row index given is out of range.");
+	}
+	else
+	{
+		return d_grid[row_idx].d_rowID;
+	}
+}
+
+
+/*************************************************************************
+	Return the zero based row index of the row with the specified ID.
+*************************************************************************/
+uint MultiColumnList::getRowWithID(uint row_id) const
+{
+	for (uint i = 0; i < getRowCount(); ++i)
+	{
+		if (d_grid[i].d_rowID == row_id)
+		{
+			return i;
+		}
+	}
+
+	// No such row found, throw exception
+	throw InvalidRequestException((utf8*)"MultiColumnList::getRowWithID - no row with the requested ID is present.");
+}
+
+
 /*************************************************************************
 	std algorithm predicate used for sorting in descending order (static)
 *************************************************************************/
@@ -2195,6 +2248,8 @@ void MultiColumnList::addMultiColumnListProperties(void)
 	addProperty(&d_sortColumnIDProperty);
 	addProperty(&d_sortDirectionProperty);
 	addProperty(&d_sortSettingProperty);
+	addProperty(&d_columnHeaderProperty);
+	addProperty(&d_rowCountProperty);
 }
 
 
@@ -2269,6 +2324,76 @@ void MultiColumnList::autoSizeColumnHeader(uint col_idx)
 }
 
 
+/*************************************************************************
+	Set the ID code assigned to a given row.
+*************************************************************************/
+void MultiColumnList::setRowID(uint row_idx, uint row_id)
+{
+	// check for invalid index
+	if (row_idx >= getRowCount())
+	{
+		throw InvalidRequestException((utf8*)"MultiColumnList::setRowID - the row index given is out of range.");
+	}
+	else
+	{
+		d_grid[row_idx].d_rowID = row_id;
+	}
+}
+
+int MultiColumnList::writePropertiesXML(OutStream& out_stream) const
+{
+    // basically this is here to translate the columns in the list into
+    // instances of the <ColumnHeader> element.  Because the SortColumnID
+    // property requires the column to exist, we also write that out manually.
+
+    // Dump all other properties first
+    int propCnt = Window::writePropertiesXML(out_stream);
+
+    // create an dump <ColumnHeader> elements
+    for (uint i = 0; i < getColumnCount(); ++i)
+    {
+        ListHeaderSegment& seg = getHeaderSegmentForColumn(i);
+
+        // start of property element,
+        String propString("<Property Name=\"ColumnHeader\" Value=\"");
+        // column text
+        propString += "text:";
+        propString += seg.getText();
+        // column width
+        propString += " width:";
+        propString += PropertyHelper::floatToString(seg.getRelativeWidth());
+        // column id
+        propString += " id:";
+        propString += PropertyHelper::uintToString(seg.getID());
+        // close the tag
+        propString += "\" />";
+
+        // write this out to the stream
+        out_stream << propString.c_str() << std::endl;
+
+        ++propCnt;
+    }
+
+    // write out SortColumnID property, if any(!)
+		try
+		{
+			uint sortColumnID = getColumnWithID(getSortColumn());
+			if (sortColumnID != 0)
+			{
+				  out_stream << "<Property Name=\"SortColumnID\" Value=\"" << PropertyHelper::uintToString(sortColumnID).c_str() << "\" />" << std::endl;
+					++propCnt;
+			}
+		}
+		catch (InvalidRequestException)
+		{
+			// This catches error(s) from the MultiLineColumnList for example
+			Logger::getSingleton().logEvent("MultiColumnList::writePropertiesXML - invalid sort column requested. Continuing...", Errors);
+		}
+
+    return propCnt;
+}
+
+
 //////////////////////////////////////////////////////////////////////////
 /*************************************************************************
 	Operators for ListRow
