@@ -70,10 +70,10 @@ const String MultiLineEditbox::EventVertScrollbarModeChanged( (utf8*)"VertScroll
 const String MultiLineEditbox::EventHorzScrollbarModeChanged( (utf8*)"HorzScrollbarModeChanged" );
 
 // default colours
-const ulong	MultiLineEditbox::DefaultNormalTextColour			= 0xFFFFFFFF;
-const ulong	MultiLineEditbox::DefaultSelectedTextColour			= 0xFF000000;
-const ulong	MultiLineEditbox::DefaultNormalSelectionColour		= 0xFF6060FF;
-const ulong	MultiLineEditbox::DefaultInactiveSelectionColour	= 0xFF808080;
+const argb_t MultiLineEditbox::DefaultNormalTextColour			= 0xFFFFFFFF;
+const argb_t MultiLineEditbox::DefaultSelectedTextColour		= 0xFF000000;
+const argb_t MultiLineEditbox::DefaultNormalSelectionColour		= 0xFF6060FF;
+const argb_t MultiLineEditbox::DefaultInactiveSelectionColour	= 0xFF808080;
 
 // Static data initialisation
 String MultiLineEditbox::d_lineBreakChars((utf8*)"\n");
@@ -125,14 +125,17 @@ MultiLineEditbox::~MultiLineEditbox(void)
 void MultiLineEditbox::initialise(void)
 {
 	// create the component sub-widgets
-	d_vertScrollbar = createVertScrollbar();
-	d_horzScrollbar = createHorzScrollbar();
+	d_vertScrollbar = createVertScrollbar(getName() + "__auto_vscrollbar__");
+	d_horzScrollbar = createHorzScrollbar(getName() + "__auto_hscrollbar__");
 
 	addChildWindow(d_vertScrollbar);
 	addChildWindow(d_horzScrollbar);
 
+    d_vertScrollbar->subscribeEvent(Scrollbar::EventScrollPositionChanged, Event::Subscriber(&MultiLineEditbox::handle_scrollChange, this));
+    d_horzScrollbar->subscribeEvent(Scrollbar::EventScrollPositionChanged, Event::Subscriber(&MultiLineEditbox::handle_scrollChange, this));
+
 	formatText();
-	layoutComponentWidgets();
+	performChildWindowLayout();
 }
 
 
@@ -148,7 +151,7 @@ bool MultiLineEditbox::hasInputFocus(void) const
 /*************************************************************************
 	return the current selection start point.	
 *************************************************************************/
-ulong MultiLineEditbox::getSelectionStartIndex(void) const
+size_t MultiLineEditbox::getSelectionStartIndex(void) const
 {
 	return (d_selectionStart != d_selectionEnd) ? d_selectionStart : d_caratPos;
 }
@@ -157,7 +160,7 @@ ulong MultiLineEditbox::getSelectionStartIndex(void) const
 /*************************************************************************
 	return the current selection end point.
 *************************************************************************/
-ulong MultiLineEditbox::getSelectionEndIndex(void) const
+size_t MultiLineEditbox::getSelectionEndIndex(void) const
 {
 	return (d_selectionStart != d_selectionEnd) ? d_selectionEnd : d_caratPos;
 }
@@ -166,7 +169,7 @@ ulong MultiLineEditbox::getSelectionEndIndex(void) const
 /*************************************************************************
 	return the length of the current selection (in code points / characters).	
 *************************************************************************/
-ulong MultiLineEditbox::getSelectionLength(void) const
+size_t MultiLineEditbox::getSelectionLength(void) const
 {
 	return d_selectionEnd - d_selectionStart;
 }
@@ -191,27 +194,21 @@ void MultiLineEditbox::addMultiLineEditboxEvents(void)
 /*************************************************************************
 	Perform the actual rendering for this Window.	
 *************************************************************************/
-void MultiLineEditbox::drawSelf(float z)
+void MultiLineEditbox::populateRenderCache()
 {
 	// get the derived class to render general stuff before we handle the text itself
-	renderEditboxBaseImagery(z);
+	cacheEditboxBaseImagery();
 
 	//
 	// Render edit box text
 	//
-	// calculate on-screen position of area we have to render into
-	Rect absarea(getTextRenderArea());
-	absarea.offset(getUnclippedPixelRect().getPosition());
+	Rect textarea(getTextRenderArea());
 
-	// calculate clipper for text rendering area
-	Rect clipper(absarea.getIntersection(getPixelRect()));
-
-	absarea.offset(Point(-d_horzScrollbar->getScrollPosition(), -d_vertScrollbar->getScrollPosition()));
-	renderTextLines(absarea, clipper);
+	cacheTextLines(textarea);
 
 	if (hasInputFocus() && !isReadOnly())
 	{
-		renderCarat(absarea.d_left, absarea.d_top, clipper);
+		cacheCaratImagery(textarea);
 	}
 
 }
@@ -236,7 +233,7 @@ void MultiLineEditbox::setReadOnly(bool setting)
 /*************************************************************************
 	Set the current position of the carat.	
 *************************************************************************/
-void MultiLineEditbox::setCaratIndex(ulong carat_pos)
+void MultiLineEditbox::setCaratIndex(size_t carat_pos)
 {
 	// make sure new position is valid
 	if (carat_pos > d_text.length() - 1)
@@ -261,7 +258,7 @@ void MultiLineEditbox::setCaratIndex(ulong carat_pos)
 /*************************************************************************
 	Define the current selection for the edit box	
 *************************************************************************/
-void MultiLineEditbox::setSelection(ulong start_pos, ulong end_pos)
+void MultiLineEditbox::setSelection(size_t start_pos, size_t end_pos)
 {
 	// ensure selection start point is within the valid range
 	if (start_pos > d_text.length() - 1)
@@ -278,7 +275,7 @@ void MultiLineEditbox::setSelection(ulong start_pos, ulong end_pos)
 	// ensure start is before end
 	if (start_pos > end_pos)
 	{
-		ulong tmp = end_pos;
+		size_t tmp = end_pos;
 		end_pos = start_pos;
 		start_pos = tmp;
 	}
@@ -301,7 +298,7 @@ void MultiLineEditbox::setSelection(ulong start_pos, ulong end_pos)
 /*************************************************************************
 	set the maximum text length for this edit box.	
 *************************************************************************/
-void MultiLineEditbox::setMaxTextLength(ulong max_len)
+void MultiLineEditbox::setMaxTextLength(size_t max_len)
 {
 	if (d_maxTextLen != max_len)
 	{
@@ -374,15 +371,13 @@ void MultiLineEditbox::ensureCaratIsVisible(void)
 {
 	// calculate the location of the carat
 	const Font* fnt = getFont();
+	size_t caratLine = getLineNumberFromIndex(d_caratPos);
 
-	uint indexCount = 0;
-	uint caratLine = getLineNumberFromIndex(d_caratPos);
-
-	if (caratLine < (uint)d_lines.size())
+	if (caratLine < d_lines.size())
 	{
 		Rect textArea(getTextRenderArea());
 
-		uint caratLineIdx = d_caratPos - d_lines[caratLine].d_startIdx;
+		size_t caratLineIdx = d_caratPos - d_lines[caratLine].d_startIdx;
 
 		float ypos = caratLine * fnt->getLineSpacing();
 		float xpos = fnt->getTextExtent(d_text.substr(d_lines[caratLine].d_startIdx, caratLineIdx));
@@ -509,117 +504,121 @@ void MultiLineEditbox::configureScrollbars(void)
 /*************************************************************************
 	Render text lines.	
 *************************************************************************/
-void MultiLineEditbox::renderTextLines(const Rect& dest_area, const Rect& clipper) const
+void MultiLineEditbox::cacheTextLines(const Rect& dest_area)
 {
-	// text is already formatted, we just grab the lines and render them with the required alignment.
-	Rect drawArea(dest_area);
-	Renderer* renderer = System::getSingleton().getRenderer();
-	const Font* fnt = getFont();
+    // text is already formatted, we just grab the lines and render them with the required alignment.
+    Rect drawArea(dest_area);
+    drawArea.offset(Point(-d_horzScrollbar->getScrollPosition(), -d_vertScrollbar->getScrollPosition()));
 
-	if (fnt != NULL)
-	{
-		// get layers to use for rendering
-		float textZ = renderer->getZLayer(4);
-		float selZ  = renderer->getZLayer(3);
+    Renderer* renderer = System::getSingleton().getRenderer();
+    const Font* fnt = getFont();
 
-		// calculate final colours to use.
-		float alpha = getEffectiveAlpha();
-		colour normalTextCol  = ((d_normalTextColour & 0x00FFFFFF) | (((ulong)(((float)(d_normalTextColour >> 24)) * alpha)) << 24));
-		colour selectTextCol  = ((d_selectTextColour & 0x00FFFFFF) | (((ulong)(((float)(d_selectTextColour >> 24)) * alpha)) << 24));
+    if (fnt)
+    {
+        // get layers to use for rendering
+        float textZ = renderer->getZLayer(4) - renderer->getCurrentZ();
+        float selZ  = renderer->getZLayer(3) - renderer->getCurrentZ();
 
-		colour selectBrushCol = hasInputFocus() ?
-			((d_selectBrushColour & 0x00FFFFFF) | (((ulong)(((float)(d_selectBrushColour >> 24)) * alpha)) << 24)) :
-			((d_inactiveSelectBrushColour & 0x00FFFFFF) | (((ulong)(((float)(d_selectBrushColour >> 24)) * alpha)) << 24));
+        // calculate final colours to use.
+        ColourRect colours;
+        float alpha = getEffectiveAlpha();
+        colour normalTextCol = d_normalTextColour;
+        normalTextCol.setAlpha(normalTextCol.getAlpha() * alpha);
+        colour selectTextCol = d_selectTextColour;
+        selectTextCol.setAlpha(selectTextCol.getAlpha() * alpha);
+        colour selectBrushCol = hasInputFocus() ? d_selectBrushColour : d_inactiveSelectBrushColour;
+        selectBrushCol.setAlpha(selectBrushCol.getAlpha() * alpha);
 
-		// for each formatted line.
-		for (uint i = 0; i < (uint)d_lines.size(); ++i)
-		{
-			Rect lineRect(drawArea);
-			const LineInfo& currLine = d_lines[i];
-			String lineText(d_text.substr(currLine.d_startIdx, currLine.d_length));
+        // for each formatted line.
+        for (size_t i = 0; i < d_lines.size(); ++i)
+        {
+            Rect lineRect(drawArea);
+            const LineInfo& currLine = d_lines[i];
+            String lineText(d_text.substr(currLine.d_startIdx, currLine.d_length));
 
-			// if it is a simple 'no selection area' case
-			if ((currLine.d_startIdx >= d_selectionEnd) ||
-				((currLine.d_startIdx + currLine.d_length) <= d_selectionStart) ||
-				(d_selectionBrush == NULL))
-			{
-				// render the complete line.
-				fnt->drawText(lineText, lineRect, textZ, clipper, LeftAligned, ColourRect(normalTextCol));
-			}
-			//
-			// we have at least some selection highlight to do
-			//
-			else
-			{
-				// Start of actual rendering section.
-				String sect;
-				uint sectIdx = 0, sectLen;
-				float selStartOffset = 0.0f, selAreaWidth = 0.0f;
+            // if it is a simple 'no selection area' case
+            if ((currLine.d_startIdx >= d_selectionEnd) ||
+                ((currLine.d_startIdx + currLine.d_length) <= d_selectionStart) ||
+                (d_selectionBrush == NULL))
+            {
+                colours.setColours(normalTextCol);
+                // render the complete line.
+                d_renderCache.cacheText(lineText, fnt, LeftAligned, lineRect, textZ, colours, &dest_area);
+            }
+            // we have at least some selection highlighting to do
+            else
+            {
+                // Start of actual rendering section.
+                String sect;
+                size_t sectIdx = 0, sectLen;
+                float selStartOffset = 0.0f, selAreaWidth = 0.0f;
 
-				// render any text prior to selected region of line.
-				if (currLine.d_startIdx < d_selectionStart)
-				{
-					// calculate length of text section
-					sectLen = d_selectionStart - currLine.d_startIdx;
+                // render any text prior to selected region of line.
+                if (currLine.d_startIdx < d_selectionStart)
+                {
+                    // calculate length of text section
+                    sectLen = d_selectionStart - currLine.d_startIdx;
 
-					// get text for this section
-					sect = lineText.substr(sectIdx, sectLen);
-					sectIdx += sectLen;
+                    // get text for this section
+                    sect = lineText.substr(sectIdx, sectLen);
+                    sectIdx += sectLen;
 
-					// get the pixel offset to the beginning of the selection area highlight.
-					selStartOffset = fnt->getTextExtent(sect);
+                    // get the pixel offset to the beginning of the selection area highlight.
+                    selStartOffset = fnt->getTextExtent(sect);
 
-					// draw this portion of the text
-					fnt->drawText(sect, lineRect, textZ, clipper, LeftAligned, ColourRect(normalTextCol));
+                    // draw this portion of the text
+                    colours.setColours(normalTextCol);
+                    d_renderCache.cacheText(sect, fnt, LeftAligned, lineRect, textZ, colours, &dest_area);
 
-					// set position ready for next portion of text
-					lineRect.d_left += selStartOffset;
-				}
+                    // set position ready for next portion of text
+                    lineRect.d_left += selStartOffset;
+                }
 
-				// calculate the length of the selected section
-				sectLen = ceguimin(d_selectionEnd - currLine.d_startIdx, currLine.d_length) - sectIdx;
+                // calculate the length of the selected section
+                sectLen = ceguimin(d_selectionEnd - currLine.d_startIdx, currLine.d_length) - sectIdx;
 
-				// get the text for this section
-				sect = lineText.substr(sectIdx, sectLen);
-				sectIdx += sectLen;
+                // get the text for this section
+                sect = lineText.substr(sectIdx, sectLen);
+                sectIdx += sectLen;
 
-				// get the extent to use as the width of the selection area highlight
-				selAreaWidth = fnt->getTextExtent(sect);
+                // get the extent to use as the width of the selection area highlight
+                selAreaWidth = fnt->getTextExtent(sect);
 
-				// draw the text for this section
-				fnt->drawText(sect, lineRect, textZ, clipper, LeftAligned, ColourRect(selectTextCol));
+                // draw the text for this section
+                colours.setColours(selectTextCol);
+                d_renderCache.cacheText(sect, fnt, LeftAligned, lineRect, textZ, colours, &dest_area);
 
-				// render any text beyond selected region of line
-				if (sectIdx < currLine.d_length)
-				{
-					// update render position to the end of the selected area.
-					lineRect.d_left += selAreaWidth;
+                // render any text beyond selected region of line
+                if (sectIdx < currLine.d_length)
+                {
+                    // update render position to the end of the selected area.
+                    lineRect.d_left += selAreaWidth;
 
-					// calculate length of this section
-					sectLen = currLine.d_length - sectIdx;
+                    // calculate length of this section
+                    sectLen = currLine.d_length - sectIdx;
 
-					// get the text for this section
-					sect = lineText.substr(sectIdx, sectLen);
+                    // get the text for this section
+                    sect = lineText.substr(sectIdx, sectLen);
 
-					// render the text for this section.
-					fnt->drawText(sect, lineRect, textZ, clipper, LeftAligned, ColourRect(normalTextCol));
-				}
+                    // render the text for this section.
+                    colours.setColours(normalTextCol);
+                    d_renderCache.cacheText(sect, fnt, LeftAligned, lineRect, textZ, colours, &dest_area);
+                }
 
-				// calculate area for the selection brush on this line
-				lineRect.d_left = drawArea.d_left + selStartOffset;
-				lineRect.d_right = lineRect.d_left + selAreaWidth;
-				lineRect.d_bottom = lineRect.d_top + fnt->getLineSpacing();
+                // calculate area for the selection brush on this line
+                lineRect.d_left = drawArea.d_left + selStartOffset;
+                lineRect.d_right = lineRect.d_left + selAreaWidth;
+                lineRect.d_bottom = lineRect.d_top + fnt->getLineSpacing();
 
-				// render the selection area brush for this line
-				d_selectionBrush->draw(lineRect, selZ, clipper, ColourRect(selectBrushCol));
-			}
-
-			// update master position for next line in paragraph.
-			drawArea.d_top += fnt->getLineSpacing();
-		}
-
-	}
+                // render the selection area brush for this line
+                colours.setColours(selectBrushCol);
+                d_renderCache.cacheImage(*d_selectionBrush, lineRect, selZ, colours, &dest_area);
+            }
 
+            // update master position for next line in paragraph.
+            drawArea.d_top += fnt->getLineSpacing();
+        }
+    }
 }
 
 
@@ -661,7 +660,7 @@ void MultiLineEditbox::formatText(void)
 			{
 				// no word wrapping, so we are just one long line.
 				line.d_startIdx = currPos;
-				line.d_length	= (uint)paraLen;
+				line.d_length	= paraLen;
 				line.d_extent	= fnt->getTextExtent(paraText); 
 				d_lines.push_back(line);
 
@@ -687,7 +686,7 @@ void MultiLineEditbox::formatText(void)
 					while (lineLen < (paraLen - lineIndex))
 					{
 						// get cp / char count of next token
-						uint nextTokenSize = getNextTokenLength(paraText, lineIndex + lineLen);
+						size_t nextTokenSize = getNextTokenLength(paraText, lineIndex + lineLen);
 
 						// get pixel width of the token
 						float tokenExtent  = fnt->getTextExtent(paraText.substr(lineIndex + lineLen, nextTokenSize));
@@ -744,23 +743,23 @@ void MultiLineEditbox::formatText(void)
 	Return the length of the next token in String 'text' starting at
 	index 'start_idx'.
 *************************************************************************/
-uint MultiLineEditbox::getNextTokenLength(const String& text, uint start_idx) const
+size_t MultiLineEditbox::getNextTokenLength(const String& text, size_t start_idx) const
 {
 	String::size_type pos = text.find_first_of(TextUtils::DefaultWrapDelimiters, start_idx);
 
 	// handle case where no more whitespace exists (so this is last token)
 	if (pos == String::npos)
 	{
-		return ((uint)text.length()) - start_idx;
+		return (text.length() - start_idx);
 	}
 	// handle 'delimiter' token cases
-	else if (((uint)pos) - start_idx == 0)
+	else if ((pos - start_idx) == 0)
 	{
 		return 1;
 	}
 	else
 	{
-		return ((uint)pos) - start_idx;
+		return (pos - start_idx);
 	}
 
 }
@@ -770,7 +769,7 @@ uint MultiLineEditbox::getNextTokenLength(const String& text, uint start_idx) co
 	Return the text code point index that is rendered closest to screen
 	position 'pt'.	
 *************************************************************************/
-uint MultiLineEditbox::getTextIndexFromPosition(const Point& pt) const
+size_t MultiLineEditbox::getTextIndexFromPosition(const Point& pt) const
 {
 	//
 	// calculate final window position to be checked
@@ -791,20 +790,20 @@ uint MultiLineEditbox::getTextIndexFromPosition(const Point& pt) const
 	wndPt.d_x += d_horzScrollbar->getScrollPosition();
 	wndPt.d_y += d_vertScrollbar->getScrollPosition();
 
-	uint lineNumber = (uint)(wndPt.d_y / getFont()->getLineSpacing());
+	size_t lineNumber = static_cast<size_t>(wndPt.d_y / getFont()->getLineSpacing());
 
-	if (lineNumber >= (uint)d_lines.size())
+	if (lineNumber >= d_lines.size())
 	{
-		lineNumber = (uint)d_lines.size() - 1;
+		lineNumber = d_lines.size() - 1;
 	}
 
 	String lineText(d_text.substr(d_lines[lineNumber].d_startIdx, d_lines[lineNumber].d_length));
 
-	uint lineIdx = getFont()->getCharAtPixel(lineText, wndPt.d_x);
+	size_t lineIdx = getFont()->getCharAtPixel(lineText, wndPt.d_x);
 
-	if (lineIdx >= (uint)lineText.length() - 1)
+	if (lineIdx >= lineText.length() - 1)
 	{
-		lineIdx = (uint)lineText.length() - 1;
+		lineIdx = lineText.length() - 1;
 	}
 
 	return d_lines[lineNumber].d_startIdx + lineIdx;
@@ -815,22 +814,22 @@ uint MultiLineEditbox::getTextIndexFromPosition(const Point& pt) const
 	Return the line number a given index falls on with the current
 	formatting.  Will return last line if index is out of range.
 *************************************************************************/
-uint MultiLineEditbox::getLineNumberFromIndex(uint index) const
+size_t MultiLineEditbox::getLineNumberFromIndex(size_t index) const
 {
-	uint lineCount = (uint)d_lines.size();
+	size_t lineCount = d_lines.size();
 
 	if (lineCount == 0)
 	{
 		return 0;
 	}
-	else if (index >= (uint)d_text.length() - 1)
+	else if (index >= d_text.length() - 1)
 	{
 		return lineCount - 1;
 	}
 	else
 	{
-		uint indexCount = 0;
-		uint caratLine = 0;
+		size_t indexCount = 0;
+		size_t caratLine = 0;
 
 		for (; caratLine < lineCount; ++caratLine)
 		{
@@ -1076,11 +1075,11 @@ void MultiLineEditbox::handleDocEnd(uint sysKeys)
 *************************************************************************/
 void MultiLineEditbox::handleLineHome(uint sysKeys)
 {
-	uint line = getLineNumberFromIndex(d_caratPos);
+	size_t line = getLineNumberFromIndex(d_caratPos);
 
-	if (line < (uint)d_lines.size())
+	if (line < d_lines.size())
 	{
-		uint lineStartIdx = d_lines[line].d_startIdx;
+		size_t lineStartIdx = d_lines[line].d_startIdx;
 
 		if (d_caratPos > lineStartIdx)
 		{
@@ -1106,11 +1105,11 @@ void MultiLineEditbox::handleLineHome(uint sysKeys)
 *************************************************************************/
 void MultiLineEditbox::handleLineEnd(uint sysKeys)
 {
-	uint line = getLineNumberFromIndex(d_caratPos);
+	size_t line = getLineNumberFromIndex(d_caratPos);
 
-	if (line < (uint)d_lines.size())
+	if (line < d_lines.size())
 	{
-		uint lineEndIdx = d_lines[line].d_startIdx + d_lines[line].d_length - 1;
+		size_t lineEndIdx = d_lines[line].d_startIdx + d_lines[line].d_length - 1;
 
 		if (d_caratPos < lineEndIdx)
 		{
@@ -1136,7 +1135,7 @@ void MultiLineEditbox::handleLineEnd(uint sysKeys)
 *************************************************************************/
 void MultiLineEditbox::handleLineUp(uint sysKeys)
 {
-	uint caratLine = getLineNumberFromIndex(d_caratPos);
+	size_t caratLine = getLineNumberFromIndex(d_caratPos);
 
 	if (caratLine > 0)
 	{
@@ -1144,7 +1143,7 @@ void MultiLineEditbox::handleLineUp(uint sysKeys)
 
 		--caratLine;
 
-		uint newLineIndex = getFont()->getCharAtPixel(d_text.substr(d_lines[caratLine].d_startIdx, d_lines[caratLine].d_length), caratPixelOffset);
+		size_t newLineIndex = getFont()->getCharAtPixel(d_text.substr(d_lines[caratLine].d_startIdx, d_lines[caratLine].d_length), caratPixelOffset);
 
 		setCaratIndex(d_lines[caratLine].d_startIdx + newLineIndex);
 	}
@@ -1166,15 +1165,15 @@ void MultiLineEditbox::handleLineUp(uint sysKeys)
 *************************************************************************/
 void MultiLineEditbox::handleLineDown(uint sysKeys)
 {
-	uint caratLine = getLineNumberFromIndex(d_caratPos);
+	size_t caratLine = getLineNumberFromIndex(d_caratPos);
 
-	if ((d_lines.size() > 1) && (caratLine < (uint)(d_lines.size() - 1)))
+	if ((d_lines.size() > 1) && (caratLine < (d_lines.size() - 1)))
 	{
 		float caratPixelOffset = getFont()->getTextExtent(d_text.substr(d_lines[caratLine].d_startIdx, d_caratPos - d_lines[caratLine].d_startIdx));
 
 		++caratLine;
 
-		uint newLineIndex = getFont()->getCharAtPixel(d_text.substr(d_lines[caratLine].d_startIdx, d_lines[caratLine].d_length), caratPixelOffset);
+		size_t newLineIndex = getFont()->getCharAtPixel(d_text.substr(d_lines[caratLine].d_startIdx, d_lines[caratLine].d_length), caratPixelOffset);
 
 		setCaratIndex(d_lines[caratLine].d_startIdx + newLineIndex);
 	}
@@ -1202,7 +1201,7 @@ void MultiLineEditbox::handleNewLine(uint sysKeys)
 		eraseSelectedText();
 
 		// if there is room
-		if ((ulong)d_text.length() - 1 < d_maxTextLen)
+		if (d_text.length() - 1 < d_maxTextLen)
 		{
 			d_text.insert(getCaratIndex(), 1, 0x0a);
 			d_caratPos++;
@@ -1291,8 +1290,8 @@ void MultiLineEditbox::onMouseTripleClicked(MouseEventArgs& e)
 
 	if (e.button == LeftButton)
 	{
-		uint caratLine = getLineNumberFromIndex(d_caratPos);
-		uint lineStart = d_lines[caratLine].d_startIdx;
+		size_t caratLine = getLineNumberFromIndex(d_caratPos);
+		size_t lineStart = d_lines[caratLine].d_startIdx;
 
 		// find end of last paragraph
 		String::size_type paraStart = d_text.find_last_of(d_lineBreakChars, lineStart);
@@ -1371,7 +1370,7 @@ void MultiLineEditbox::onCharacter(KeyEventArgs& e)
 		eraseSelectedText();
 
 		// if there is room
-		if ((ulong)d_text.length() - 1 < d_maxTextLen)
+		if (d_text.length() - 1 < d_maxTextLen)
 		{
 			d_text.insert(getCaratIndex(), 1, e.codepoint);
 			d_caratPos++;
@@ -1477,6 +1476,10 @@ void MultiLineEditbox::onKeyDown(KeyEventArgs& e)
 				handleLineEnd(e.sysKeys);
 			}
 			break;
+
+        // default case is now to leave event as (possibly) unhandled.
+        default:
+            return;
 		}
 
 		e.handled = true;
@@ -1490,29 +1493,27 @@ void MultiLineEditbox::onKeyDown(KeyEventArgs& e)
 *************************************************************************/
 void MultiLineEditbox::onTextChanged(WindowEventArgs& e)
 {
-	// ensure last character is a new line
-	if ((d_text.length() == 0) || (d_text[d_text.length() - 1] != '\n'))
-	{
-		d_text.append(1, '\n');
-	}
+    // ensure last character is a new line
+    if ((d_text.length() == 0) || (d_text[d_text.length() - 1] != '\n'))
+        d_text.append(1, '\n');
 
-	// base class processing
-	Window::onTextChanged(e);
+    // base class processing
+    Window::onTextChanged(e);
 
-	// clear selection
-	clearSelection();
+    // clear selection
+    clearSelection();
+    // layout new text
+    formatText();
+    // layout child windows (scrollbars) since text layout may have changed
+    performChildWindowLayout();
+    // ensure carat is still within the text
+    setCaratIndex(getCaratIndex());
+    // ensure carat is visible
+    // NB: this will already have been called at least once, but since we
+    // may have changed the formatting of the text, it needs to be called again.
+    ensureCaratIsVisible();
 
-	// make sure carat is within the text
-	if (getCaratIndex() > d_text.length() - 1)
-	{
-		setCaratIndex(d_text.length() - 1);
-	}
-
-	formatText();
-	layoutComponentWidgets();
-	ensureCaratIsVisible();
-
-	e.handled = true;
+    e.handled = true;
 }
 
 
@@ -1521,12 +1522,11 @@ void MultiLineEditbox::onTextChanged(WindowEventArgs& e)
 *************************************************************************/
 void MultiLineEditbox::onSized(WindowEventArgs& e)
 {
+	formatText();
+
 	// base class handling
 	Window::onSized(e);
 
-	formatText();
-	layoutComponentWidgets();
-
 	e.handled = true;
 }
 
@@ -1656,4 +1656,16 @@ void MultiLineEditbox::addMultiLineEditboxProperties(void)
 	addProperty(&d_inactiveSelectionColourProperty);
 }
 
+/*************************************************************************
+    Handler for scroll position changes.
+*************************************************************************/
+bool MultiLineEditbox::handle_scrollChange(const EventArgs& args)
+{
+    // simply trigger a redraw of the Listbox.
+    requestRedraw();
+    return true;
+}
+
+
+
 } // End of  CEGUI namespace section
