@@ -58,8 +58,8 @@ const String TabControl::EventSelectionChanged( (utf8*)"TabSelectionChanged" );
 *************************************************************************/
 TabControl::TabControl(const String& type, const String& name)
 	: Window(type, name),
-	d_tabContentPane(NULL),
     d_tabButtonPane(NULL),
+    d_tabContentPane(NULL),
     d_nextTabIndex(0)
 {
 	addTabControlEvents();
@@ -83,13 +83,13 @@ TabControl::~TabControl(void)
 void TabControl::initialise(void)
 {
 	// create the component sub-widgets
-	d_tabContentPane = createTabContentPane();
-    d_tabButtonPane = createTabButtonPane();
+	d_tabContentPane = createTabContentPane(getName() + "__auto_TabPane__");
+    d_tabButtonPane = createTabButtonPane(getName() + "__auto_TabPane__Buttons");
 
 	addChildWindow(d_tabContentPane);
     addChildWindow(d_tabButtonPane);
 
-	layoutComponentWidgets();
+	performChildWindowLayout();
 }
 /*************************************************************************
 Get the number of tabs
@@ -117,7 +117,7 @@ Get the tab for the given index
 *************************************************************************/
 Window*	TabControl::getTabContentsAtIndex(uint index) const
 {
-	return d_tabContentPane->getChildAtIdx(index);
+    return d_tabButtonIndexMap.find(index)->second->getTargetWindow();
 }
 
 /*************************************************************************
@@ -134,15 +134,16 @@ Return whether the tab content window is currently selected.
 *************************************************************************/
 uint TabControl::getSelectedTabIndex() const
 {
-	uint index = 0;
+    uint index;
     TabButtonIndexMap::const_iterator i, iend;
     iend = d_tabButtonIndexMap.end();
-    for (i = d_tabButtonIndexMap.begin(); i != iend; ++i, ++index)
+    for (i = d_tabButtonIndexMap.begin(); i != iend; ++i)
     {
         // get corresponding tab button and content window
         TabButton* tb = i->second;
         if (tb->isSelected())
         {
+            index = i->first;
 			break;
         }
 	}
@@ -201,7 +202,7 @@ void TabControl::setRelativeTabHeight(float height)
     d_rel_tabHeight = height;
     d_abs_tabHeight = relativeToAbsoluteY(height);
 
-    layoutComponentWidgets();
+    performChildWindowLayout();
 }
 /*************************************************************************
 Set the tab height
@@ -211,7 +212,7 @@ void TabControl::setAbsoluteTabHeight(float height)
     d_abs_tabHeight = height;
     d_rel_tabHeight = absoluteToRelativeY(height);
 
-    layoutComponentWidgets();
+    performChildWindowLayout();
 }
 /*************************************************************************
 Set the tab height
@@ -251,7 +252,7 @@ void TabControl::setRelativeTabTextPadding(float height)
     d_rel_tabPadding = height;
     d_abs_tabPadding = relativeToAbsoluteY(height);
 
-    layoutComponentWidgets();
+    performChildWindowLayout();
 }
 /*************************************************************************
 Set the tab text padding
@@ -261,7 +262,7 @@ void TabControl::setAbsoluteTabTextPadding(float height)
     d_abs_tabPadding = height;
     d_rel_tabPadding = absoluteToRelativeY(height);
 
-    layoutComponentWidgets();
+    performChildWindowLayout();
 }
 /*************************************************************************
 Set the tab text padding
@@ -298,7 +299,7 @@ void TabControl::addTab(Window* wnd)
         wnd->setVisible(false);
     }
     // Just request redraw
-    layoutComponentWidgets();
+    performChildWindowLayout();
     requestRedraw();
     // Subscribe to text changed event so that we can resize as needed
     wnd->subscribeEvent(Window::EventTextChanged, 
@@ -310,6 +311,10 @@ Remove a tab
 *************************************************************************/
 void TabControl::removeTab(const String& name)
 {
+    // do nothing if given window is not attached as a tab.
+    if (!d_tabContentPane->isChild(name))
+        return;
+
     Window* wnd = d_tabContentPane->getChild(name);
     // Was this selected?
     bool reselect = wnd->isVisible();
@@ -328,16 +333,19 @@ void TabControl::removeTab(const String& name)
         }
     }
 
-    layoutComponentWidgets();
+    performChildWindowLayout();
 
     requestRedraw();
-
 }
 /*************************************************************************
 Remove a tab by ID
 *************************************************************************/
 void TabControl::removeTab(uint ID)
 {
+    // do nothing if given window is not attached as a tab.
+    if (!d_tabContentPane->isChild(ID))
+        return;
+
     Window* wnd = d_tabContentPane->getChild(ID);
     // Was this selected?
     bool reselect = wnd->isVisible();
@@ -356,7 +364,7 @@ void TabControl::removeTab(uint ID)
         }
     }
 
-    layoutComponentWidgets();
+    performChildWindowLayout();
 
     requestRedraw();
 
@@ -522,6 +530,26 @@ void TabControl::addChild_impl(Window* wnd)
     }
 }
 /*************************************************************************
+Internal version of removing a child window
+*************************************************************************/
+void TabControl::removeChild_impl(Window* wnd)
+{
+    // protect against possible null pointers
+    if (!wnd) return;
+
+    // Look for __auto_TabPane__ in the name (hopefully no-one will use this!)
+    if (wnd->getName().find((const utf8*)"__auto_TabPane__") != String::npos)
+    {
+        // perform normal removeChild
+        Window::removeChild_impl(wnd);
+    }
+    else
+    {
+        // This is some user window, therefore remove as a tab
+        removeTab(wnd->getName());
+    }
+}
+/*************************************************************************
 Selection changed event
 *************************************************************************/
 void TabControl::onSelectionChanged(WindowEventArgs& e)
@@ -550,22 +578,12 @@ void TabControl::addTabControlEvents(void)
     addEvent(EventSelectionChanged);
 }
 /*************************************************************************
-Handler for when widget is re-sized
-*************************************************************************/
-void TabControl::onSized(WindowEventArgs& e)
-{
-    // base class processing
-    Window::onSized(e);
-
-    layoutComponentWidgets();
-
-    e.handled = true;
-}
-/*************************************************************************
 Layout the widgets
 *************************************************************************/
-void TabControl::layoutComponentWidgets(void)
+void TabControl::performChildWindowLayout()
 {
+    Window::performChildWindowLayout();
+
     if (d_tabButtonPane)
     {
         // Set the size of the tab button area (full width, height from tab height)
@@ -592,11 +610,9 @@ void TabControl::layoutComponentWidgets(void)
 /*************************************************************************
 Create tab button pane
 *************************************************************************/
-Window*	TabControl::createTabButtonPane(void) const
+Window*	TabControl::createTabButtonPane(const String& name) const
 {
-    // Generate name based on own name
-    String newName = getName() + (utf8*)"__auto_TabPane__Buttons";
-	return WindowManager::getSingleton().createWindow(GUISheet::WidgetTypeName, newName);
+	return WindowManager::getSingleton().createWindow(GUISheet::WidgetTypeName, name);
 }
 /*************************************************************************
 Text changed on a content window
@@ -609,7 +625,7 @@ bool TabControl::handleContentWindowTextChanged(const EventArgs& args)
         makeButtonName(wargs.window));
     tabButton->setText(wargs.window->getText());
     // sort out the layout
-    layoutComponentWidgets();
+    performChildWindowLayout();
 	requestRedraw();
 
 	return true;
@@ -625,4 +641,21 @@ bool TabControl::handleTabButtonClicked(const EventArgs& args)
 
 	return true;
 }
+
+int TabControl::writeChildWindowsXML(OutStream& out_stream) const
+{
+    int childOutputCount = Window::writeChildWindowsXML(out_stream);
+
+    // since TabControl content is actually added to the component tab
+    // content pane window, this overridden function exists to dump those
+    // out as if they were our own children.
+    for (uint i = 0; i < getTabCount(); ++i)
+    {
+        getTabContentsAtIndex(i)->writeXMLToStream(out_stream);
+        ++childOutputCount;
+    }
+
+    return childOutputCount;
+}
+
 } // End of  CEGUI namespace section
