@@ -57,6 +57,8 @@ OpenGLRenderer::OpenGLRenderer(uint max_quads) :
 	d_display_area.d_top	= 0;
 	d_display_area.d_right	= (float)vp[2];
 	d_display_area.d_bottom	= (float)vp[3];
+
+    setModuleIdentifierString();
 }
 
 
@@ -74,6 +76,8 @@ OpenGLRenderer::OpenGLRenderer(uint max_quads,int width, int height) :
 	d_display_area.d_top	= 0;
 	d_display_area.d_right	= static_cast<float>(width);
 	d_display_area.d_bottom	= static_cast<float>(height);
+
+    setModuleIdentifierString();
 }
 
 
@@ -497,13 +501,71 @@ void OpenGLRenderer::renderQuadDirect(const Rect& dest_rect, float z, const Text
 long OpenGLRenderer::colourToOGL(const colour& col) const
 {
 	ulong cval;
+#ifdef __BIG_ENDIAN__
+    cval =  (static_cast<ulong>(255 * col.getAlpha()));
+    cval |= (static_cast<ulong>(255 * col.getBlue())) << 8;
+    cval |= (static_cast<ulong>(255 * col.getGreen())) << 16;
+    cval |= (static_cast<ulong>(255 * col.getRed())) << 24;
+#else
 	cval =	(static_cast<ulong>(255 * col.getAlpha())) << 24;
 	cval |=	(static_cast<ulong>(255 * col.getBlue())) << 16;
 	cval |=	(static_cast<ulong>(255 * col.getGreen())) << 8;
 	cval |= (static_cast<ulong>(255 * col.getRed()));
-
+#endif
 	return cval;
 }
 
+
+/*************************************************************************
+	Set the size of the display in pixels.	
+*************************************************************************/
+void OpenGLRenderer::setDisplaySize(const Size& sz)
+{
+	if (d_display_area.getSize() != sz)
+	{
+		d_display_area.setSize(sz);
+
+		EventArgs args;
+		fireEvent(EventDisplaySizeChanged, args, EventNamespace);
+	}
+
+}
+
+void OpenGLRenderer::setModuleIdentifierString()
+{
+    // set ID string
+    d_identifierString = "CEGUI::OpenGLRenderer - Official OpenGL based renderer module for CEGUI";
+}
+
+
+/************************************************************************
+    Grabs all loaded textures to local buffers and frees them
+*************************************************************************/
+void OpenGLRenderer::grabTextures()
+{
+    typedef std::list<OpenGLTexture*> texlist;
+    texlist::iterator i = d_texturelist.begin();
+    while (i!=d_texturelist.end())
+    {
+        (*i)->grabTexture();
+        i++;
+    }
+}
+
+
+/************************************************************************
+    Restores all textures from the previous call to 'grabTextures'
+*************************************************************************/
+void OpenGLRenderer::restoreTextures()
+{
+    typedef std::list<OpenGLTexture*> texlist;
+    texlist::iterator i = d_texturelist.begin();
+    while (i!=d_texturelist.end())
+    {
+        (*i)->restoreTexture();
+        i++;
+    }
+}
+
 } // End of  CEGUI namespace section
 
