@@ -48,7 +48,8 @@ namespace CEGUI
 	Constructor
 *************************************************************************/
 OpenGLTexture::OpenGLTexture(Renderer* owner) :
-	Texture(owner)
+	Texture(owner),
+    d_grabBuffer(0)
 {
 	// generate a OGL texture that we will use.
 	glGenTextures(1, &d_ogltexture);
@@ -73,8 +74,16 @@ OpenGLTexture::OpenGLTexture(Renderer* owner) :
 *************************************************************************/
 OpenGLTexture::~OpenGLTexture(void)
 {
-	// delete OGL texture associated with this object.
-	glDeleteTextures(1, &d_ogltexture);
+    // if the grabbuffer is not empty then free it
+    if (d_grabBuffer)
+    {
+        delete [] d_grabBuffer;
+    }
+    // otherwise delete OGL texture associated with this object.
+    else
+    {
+        glDeleteTextures(1, &d_ogltexture);
+    }
 }
 
 
@@ -95,8 +104,8 @@ void OpenGLTexture::loadFromFile(const String& filename, const String& resourceG
 
 	if (img != 0)
 	{
-		d_width = img->sizeX;
-		d_height = img->sizeY;
+		d_width = static_cast<ushort>(img->sizeX);
+		d_height = static_cast<ushort>(img->sizeY);
 
 		// flip the image...
 		flipImageTGA(img);
@@ -163,6 +172,9 @@ void OpenGLTexture::loadFromFile(const String& filename, const String& resourceG
 	}
 
 # endif
+    // unload file data buffer
+	System::getSingleton().getResourceProvider()->unloadRawDataContainer(texFile);
+
 }
 
 
@@ -174,8 +186,8 @@ void OpenGLTexture::loadFromMemory(const void* buffPtr, uint buffWidth, uint buf
 	glBindTexture(GL_TEXTURE_2D, d_ogltexture);
 	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, buffWidth, buffHeight, 0, GL_RGBA ,GL_UNSIGNED_BYTE, buffPtr);
 
-	d_width  = buffWidth;
-	d_height = buffHeight;
+	d_width  = static_cast<ushort>(buffWidth);
+	d_height = static_cast<ushort>(buffHeight);
 }
 
 
@@ -209,7 +221,7 @@ void OpenGLTexture::setOGLTextureSize(uint size)
 	// delete buffer
 	delete[] buff;
 
-	d_height = d_width = size;
+	d_height = d_width = static_cast<ushort>(size);
 }
 
 
@@ -257,7 +269,7 @@ void OpenGLTexture::flipImageTGA(OpenGLTexture::tImageTGA* img)
 /////
 ///////////////////////////////// LOAD TGA \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
 
-OpenGLTexture::tImageTGA* OpenGLTexture::LoadTGA(const unsigned char* buffer, unsigned int buffer_size)
+OpenGLTexture::tImageTGA* OpenGLTexture::LoadTGA(const unsigned char* buffer, size_t buffer_size)
 {
 	tImageTGA *pImageData = NULL;		// This stores our important image data
 	short width = 0, height = 0;			// The dimensions of the image
@@ -304,6 +316,10 @@ OpenGLTexture::tImageTGA* OpenGLTexture::LoadTGA(const unsigned char* buffer, un
 	buffer += sizeof(short);
 	memcpy(&height, buffer, sizeof(short));
 	buffer += sizeof(short);
+#ifdef __BIG_ENDIAN__
+    width = ((width & 0xFF00)>>8) | ((width & 0x00FF)<<8);
+    height = ((height & 0xFF00)>>8) | ((height & 0x00FF)<<8);
+#endif
 	memcpy(&bits, buffer, sizeof(GLbyte));
 	buffer += sizeof(GLbyte);
 
@@ -336,7 +352,7 @@ OpenGLTexture::tImageTGA* OpenGLTexture::LoadTGA(const unsigned char* buffer, un
 				// files are stored as BGR instead of RGB (or use GL_BGR_EXT verses GL_RGB)
 				for(i = 0; i < stride; i += channels)
 				{
-					int temp     = pLine[i];
+					unsigned char temp     = pLine[i];
 					pLine[i]     = pLine[i + 2];
 					pLine[i + 2] = temp;
 				}
@@ -346,7 +362,7 @@ OpenGLTexture::tImageTGA* OpenGLTexture::LoadTGA(const unsigned char* buffer, un
 		else if(bits == 16)
 		{
 			unsigned short pixels = 0;
-			int r=0, g=0, b=0;
+			unsigned char r=0, g=0, b=0;
 
 			// Since we convert 16-bit images to 24 bit, we hardcode the channels to 3.
 			// We then calculate the stride and allocate memory for the pixels.
@@ -366,9 +382,9 @@ OpenGLTexture::tImageTGA* OpenGLTexture::LoadTGA(const unsigned char* buffer, un
 				// 0x1f = 11111 in binary, so since 5 bits are reserved in
 				// each unsigned short for the R, G and B, we bit shift and mask
 				// to find each value.  We then bit shift up by 3 to get the full color.
-				b = (pixels & 0x1f) << 3;
-				g = ((pixels >> 5) & 0x1f) << 3;
-				r = ((pixels >> 10) & 0x1f) << 3;
+				b = static_cast<unsigned char>((pixels & 0x1f) << 3);
+				g = static_cast<unsigned char>(((pixels >> 5) & 0x1f) << 3);
+				r = static_cast<unsigned char>(((pixels >> 10) & 0x1f) << 3);
 
 				// This essentially assigns the color to our array and swaps the
 				// B and R values at the same time.
@@ -495,4 +511,43 @@ OpenGLTexture::tImageTGA* OpenGLTexture::LoadTGA(const unsigned char* buffer, un
 }
 #endif
 
+
+/************************************************************************
+    Grabs this texture's image data into a buffer for later restoration
+*************************************************************************/
+void OpenGLTexture::grabTexture(void)
+{
+    // bind the texture we want to grab
+    glBindTexture(GL_TEXTURE_2D, d_ogltexture);
+    // allocate the buffer for storing the image data
+    d_grabBuffer = new uint8[4*d_width*d_height];
+    glGetTexImage(GL_TEXTURE_2D,0,GL_RGBA,GL_UNSIGNED_BYTE,d_grabBuffer);
+    // delete the texture
+    glDeleteTextures(1, &d_ogltexture);
+}
+
+
+/************************************************************************
+    Restores this texture from the previously grabbed image data
+*************************************************************************/
+void OpenGLTexture::restoreTexture(void)
+{
+    // recreate texture
+    // generate a OGL texture that we will use.
+    glGenTextures(1, &d_ogltexture);
+
+    // set some parameters for this texture.
+    glBindTexture(GL_TEXTURE_2D, d_ogltexture);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, 0x812F);  // GL_CLAMP_TO_EDGE
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, 0x812F);  // GL_CLAMP_TO_EDGE
+    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, d_width, d_height, 0, GL_RGBA, GL_UNSIGNED_BYTE, d_grabBuffer);
+    
+    // free the grabbuffer
+    delete [] d_grabBuffer;
+    d_grabBuffer = 0;
+}
+
 } // End of  CEGUI namespace section
